@@ -22,6 +22,7 @@ import gobject
 import locale
 import pango
 from pynicotine import slskmessages
+from pynicotine import pluginsystem
 from utils import InitialiseColumns, AppendLine, PopupMenu, FastListModel, string_sort_func, WriteLog, int_sort_func, Humanize, HumanSpeed, expand_alias, is_alias, EncodingsMenu, SaveEncoding, PressHeader, fixpath, IconNotebook
 from pynicotine.utils import _
 from ticker import Ticker
@@ -58,8 +59,12 @@ class RoomsControl:
 		self.rooms = []
 		config = self.frame.np.config.sections
 		self.PrivateRooms =  config["private_rooms"]["rooms"]
-			
-		#self.PrivateRoomsOperator = config["private_rooms"]["operator"]
+		# Config cleanup
+		for room,data in self.PrivateRooms.items():
+			if "owner" not in data:
+				self.PrivateRooms[room]["owner"] = None
+			if "operator" in data:
+				del self.PrivateRooms[room]["operator"]
 		#self.PrivateRoomsMembership = config["private_rooms"]["membership"]
 		
 		self.clist = []
@@ -68,7 +73,7 @@ class RoomsControl:
 		
 		self.cols = InitialiseColumns(frame.roomlist.RoomsList,
 			[_("Room"), 150, "text", self.RoomStatus],
-			[_("Users"), -1, "text", self.RoomStatus],
+			[_("Users"), -1, "number", self.RoomStatus],
 		)
 		self.cols[0].set_sort_column_id(0)
 		self.cols[1].set_sort_column_id(1)
@@ -106,7 +111,7 @@ class RoomsControl:
 		frame.roomlist.RoomsList.connect("button_press_event", self.OnListClicked)
 		frame.roomlist.RoomsList.set_headers_clickable(True)
 
-		frame.roomlist.HideRoomList.connect("clicked", self.OnHideRoomList)
+		frame.roomlist.HideRoomList.connect("clicked", self.OnShowRoomList)
 
 		ChatNotebook.connect("switch-page", self.OnSwitchPage)
 		try:
@@ -121,7 +126,7 @@ class RoomsControl:
 	def IsPrivateRoomOwned(self, room):
 		if room in self.PrivateRooms:
 			#print self.PrivateRooms[room]
-			if self.PrivateRooms[room]["owned"] == True:
+			if self.PrivateRooms[room]["owner"] == self.frame.np.config.sections["server"]["login"]:
 				return True
 		return False
 		
@@ -132,7 +137,7 @@ class RoomsControl:
 	
 	def IsPrivateRoomOperator(self, room):
 		if room in self.PrivateRooms:
-			if self.PrivateRooms[room]["operator"] == True:
+			if self.frame.np.config.sections["server"]["login"] in self.PrivateRooms[room]["operators"]:
 				return True
 		return False
 		
@@ -194,7 +199,7 @@ class RoomsControl:
 			if room.Main == page:
 				gobject.idle_add(room.ChatEntry.grab_focus)
 				# Remove hilite
-				self.frame.ClearNotification("rooms", None, name)
+				self.frame.Notifications.Clear("rooms", None, name)
 					
 	def ClearNotifications(self):
 		if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.hpaned1):
@@ -203,18 +208,18 @@ class RoomsControl:
 		for name, room in self.joinedrooms.items():
 			if room.Main == page:
 				# Remove hilite
-				self.frame.ClearNotification("rooms", None, name)
+				self.frame.Notifications.Clear("rooms", None, name)
 				
 	def Focused(self, page, focused):
 		if not focused:
 			return
 		for name, room in self.users.items():
 			if room.Main == page:
-				self.frame.ClearNotification("rooms", name)
+				self.frame.Notifications.Clear("rooms", name)
 			
-	def OnHideRoomList(self, widget):
+	def OnShowRoomList(self, widget):
 
-		self.frame.hide_room_list1.set_active(1)
+		self.frame.show_room_list1.set_active(1)
 		
 	def OnListClicked(self, widget, event):
 		if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
@@ -295,7 +300,7 @@ class RoomsControl:
 		if self.IsPrivateRoomMember(self.popup_room):
 			self.frame.np.queue.put(slskmessages.PrivateRoomDismember(self.popup_room))
 			del self.PrivateRooms[self.popup_room]
-			self.SetPrivateRooms(msg.privaterooms)
+			self.SetPrivateRooms()
 			
 	def OnPopupLeave(self, widget):
 		self.frame.np.queue.put(slskmessages.LeaveRoom(self.popup_room))
@@ -309,6 +314,8 @@ class RoomsControl:
 			return
 		tab = ChatRoom(self, msg.room, msg.users)
 		self.joinedrooms[msg.room] = tab
+		if msg.private is not None:
+			self.CreatePrivateRoom(msg.room, msg.owner, msg.operators)
 		angle = 0
 		try:
 			angle = int(self.frame.np.config.sections["ui"]["labelrooms"])
@@ -358,7 +365,7 @@ class RoomsControl:
 	def SetPrivateRooms(self, rooms=[]):
 		for room in rooms:
 			if room[0] not in self.PrivateRooms.keys():
-				self.PrivateRooms[room[0]] = {"users": [], "joined": room[1], "operators": [], "owned":True, "operator":False}
+				self.PrivateRooms[room[0]] = {"users": [], "joined": room[1], "operators": [],  "owner": None}
 		iter = self.roomsmodel.get_iter_root()
 		while iter is not None:
 			room = self.roomsmodel.get_value(iter, 0)
@@ -374,10 +381,15 @@ class RoomsControl:
 			elif self.IsPrivateRoomMember(room):
 				self.roomsmodel.prepend([room, num, 1])
 				
-	def CreatePrivateRoom(self, room):
+	def CreatePrivateRoom(self, room, owner=None, operators=[]):
 		if room in self.PrivateRooms:
+			if operators is not None:
+				for operator in operators:
+					if operator not in self.PrivateRooms[room]["operators"]:
+						self.PrivateRooms[room]["operators"].append(operator)
+			self.PrivateRooms[room]["owner"] = owner
 			return
-		self.PrivateRooms[room] = {"users": [], "joined": 0, "operators": [], "owned":False, "operator":False}
+		self.PrivateRooms[room] = {"users": [], "joined": 0, "operators": operators, "owned":False, "owner":owner}
 		
 	def PrivateRoomUsers(self, msg):
 		rooms = self.PrivateRooms
@@ -419,14 +431,16 @@ class RoomsControl:
 	def PrivateRoomOperatorAdded(self, msg):
 		rooms = self.PrivateRooms
 		if msg.room in rooms.keys():
-			rooms[msg.room]["operator"] = True
+			if self.frame.np.config.sections["server"]["login"] not in rooms[msg.room]["operators"]:
+				rooms[msg.room]["operators"].append(self.frame.np.config.sections["server"]["login"])
 		
 		self.SetPrivateRooms()
 
 	def PrivateRoomOperatorRemoved(self, msg):
 		rooms = self.PrivateRooms
 		if msg.room in rooms.keys():
-			rooms[msg.room]["operator"] = False
+			if self.frame.np.config.sections["server"]["login"] in rooms[msg.room]["operators"]:
+				rooms[msg.room]["operators"].remove(self.frame.np.config.sections["server"]["login"])
 
 		self.SetPrivateRooms()
 
@@ -435,8 +449,6 @@ class RoomsControl:
 		if msg.room in rooms.keys():
 			if msg.user not in rooms[msg.room]["operators"]:
 				rooms[msg.room]["operators"].append(msg.user)
-			if msg.user == self.frame.np.config.sections["server"]["login"]:
-				rooms[msg.room]["operator"] = True
 		self.SetPrivateRooms()
 
 	def PrivateRoomRemoveOperator(self, msg):
@@ -444,8 +456,7 @@ class RoomsControl:
 		if msg.room in rooms.keys():
 			if msg.user in rooms[msg.room]["operators"]:
 				rooms[msg.room]["operators"].remove(msg.user)
-			if msg.user == self.frame.np.config.sections["server"]["login"]:
-				rooms[msg.room]["operator"] = False
+
 		self.SetPrivateRooms()
 
 	def PrivateRoomAdded(self, msg):
@@ -454,6 +465,7 @@ class RoomsControl:
 		if room not in rooms:
 			self.CreatePrivateRoom(room)
 			#rooms[room]["operator"] = True
+			self.frame.logMessage(_("You have been added to a private room: %(room)s") % {"room":room} )
 		self.SetPrivateRooms()
 
 		
@@ -471,7 +483,8 @@ class RoomsControl:
 		
 	def PrivateRoomDisown(self, msg):
 		if msg.room in self.PrivateRooms.keys():
-			self.PrivateRooms[msg.room]["owner"] = False
+			if self.PrivateRooms[msg.room]["owner"] == self.frame.np.config.sections["server"]["login"]:
+				self.PrivateRooms[msg.room]["owner"] = None
 	
 	def GetUserStats(self, msg):
 		for room in self.joinedrooms.values():
@@ -518,18 +531,19 @@ class RoomsControl:
 			if self.frame.np.ipIgnored(ip):
 				#print "ignored message from IP:", ip, msg.user
 				return
-			self.joinedrooms[msg.room].SayChatRoom(msg, text)
+		self.joinedrooms[msg.room].SayChatRoom(msg, text)
 	def PublicRoomMessage(self, msg, text):
 		try:
 			room = self.joinedrooms['Public ']
 		except KeyError:
 			return
-		msg.user = "%s | %s" % (msg.room, msg.user)
-		room.SayChatRoom(msg, text)
+		#msg.user = "%s | %s" % (msg.room, msg.user)
+		room.SayChatRoom(msg, text, public=True)
 	
 	def UpdateColours(self):
 		self.frame.SetTextBG(self.frame.roomlist.RoomsList)
 		self.frame.SetTextBG(self.frame.roomlist.CreateRoomEntry)
+		self.frame.SetTextBG(self.frame.roomlist.SearchRooms)
 
 		for room in self.joinedrooms.values():
 			room.ChangeColours()
@@ -554,7 +568,7 @@ class RoomsControl:
 		del self.joinedrooms[msg.room]
 		if msg.room[-1:] != ' ': # meta rooms
 			self.frame.RoomSearchCombo_List.remove(self.frame.searchroomslist[msg.room])
-		if self.joinedrooms == {} and self.frame.hide_room_list1.get_active():
+		if self.joinedrooms == {} and not self.frame.show_room_list1.get_active():
 			win = OptionDialog(self.frame, _("You aren't in any chat rooms.") + " " + _("Open Room List?"), modal=True, status=None, option=False, third="")
 			win.connect("response", self.frame.onOpenRoomList)
 			win.set_title(_("Nicotine+")+": "+_("Open Room List?"))
@@ -644,7 +658,7 @@ class ChatRoom:
 	def __init__(self, roomsctrl, room, users, meta = False):
 		self.roomsctrl = roomsctrl
 		self.frame = roomsctrl.frame
-		self.tooltips = self.frame.tooltips
+		#self.tooltips = self.frame.tooltips
 
 		gtk.glade.set_custom_handler(self.get_custom_widget)
 		
@@ -658,8 +672,8 @@ class ChatRoom:
 	
 		self.wTree.signal_autoconnect(self)
 		
-		if not self.frame.np.config.sections["ui"]["tooltips"]:
-			self.tooltips.disable()
+		#if not self.frame.np.config.sections["ui"]["tooltips"]:
+		#	self.tooltips.disable()
 		self.room = room
 		self.lines = []
 		self.logfile = None
@@ -669,7 +683,7 @@ class ChatRoom:
 		if not config["ticker"]["hide"]:
 			self.Ticker.show()
 		
-		self.OnHideChatButtons(hide=config["ui"]["chat_hidebuttons"])
+		self.OnShowChatButtons(show=(not config["ui"]["chat_hidebuttons"]))
 			
 		if self.frame.translux:
 			self.tlux_roomlog = lambda: self.RoomLog.get_window(gtk.TEXT_WINDOW_TEXT)
@@ -725,16 +739,18 @@ class ChatRoom:
 		completion.connect("match-selected", self.frame.EntryCompletionFoundMatch, self.ChatEntry)
 		
 		self.Log.set_active(config["logging"]["chatrooms"])
-		
+		if not self.Log.get_active():
+			self.Log.set_active((self.room in config["logging"]["rooms"]))
+
 		if room in config["server"]["autojoin"]:
 			self.AutoJoin.set_active(True)
-			
+		statusiconwidth = self.frame.images["offline"].get_width()+4
 		self.cols = cols = InitialiseColumns(self.UserList, 
-			[_("Status"), 20, "pixbuf"],
+			[_("Status"), statusiconwidth, "pixbuf"],
 			[_("Country"), 25, "pixbuf"],
 			[_("User"), 100, "text", self.UserColumnDraw],
-			[_("Speed"), 0, "text", self.frame.CellDataFunc],
-			[_("Files"), 0, "text", self.frame.CellDataFunc],
+			[_("Speed"), 0, "number", self.frame.CellDataFunc],
+			[_("Files"), 0, "number", self.frame.CellDataFunc],
 		)
 		cols[0].set_sort_column_id(5)
 		cols[1].set_sort_column_id(8)
@@ -802,6 +818,22 @@ class ChatRoom:
 			("#" + _("Sear_ch this user's files"), popup.OnSearchUser, gtk.STOCK_FIND),
 			(1, _("Private rooms"), self.popup_menu_privaterooms, self.OnPrivateRooms),
 		)
+		items = self.popup_menu.get_children()
+		self.Menu_SendMessage = items[2]
+		self.Menu_ShowIPaddress = items[4]
+		self.Menu_GetUserInfo = items[5]
+		self.Menu_BrowseUser = items[6]
+		self.Menu_GivePrivileges = items[7]
+		self.Menu_AddToList = items[9]
+		self.Menu_BanUser = items[10]
+		self.Menu_IgnoreUser = items[11]
+		self.Menu_BlockUser = items[12]
+		self.Menu_IgnoreIP = items[13]
+		self.Menu_SearchUser = items[15]
+		self.Menu_PrivateRooms = items[16]
+		img = gtk.Image()
+		img.set_from_pixbuf(self.frame.images["money"])
+		self.Menu_GivePrivileges.set_image(img)
 		self.UserList.connect("button_press_event", self.OnPopupMenu)
 
 		self.ChatEntry.grab_focus()
@@ -850,7 +882,7 @@ class ChatRoom:
 			loglines = logfile.split("\n")
 			for l in loglines[ - roomlines : -1 ]:
 				# Try to parse line for username
-				if l[10].isspace() and l[11].isdigit() and l[20] in ("[", "*"):
+				if len(l) > 20 and l[10].isspace() and l[11].isdigit() and l[20] in ("[", "*"):
 					line = l[11:]
 					if l[20] == "[" and l[20:].find("] ") != -1:
 						namepos = l[20:].find("] ")
@@ -887,13 +919,23 @@ class ChatRoom:
 			gobject.idle_add(self.frame.ScrollBottom, self.ChatScroll.get_parent())
 		except IOError, e:
 			pass
-		
-	def OnFindLogWindow(self, widget):
 
-		self.frame.OnFindTextview(widget, self.RoomLog)
+	def on_key_press_event(self, widget, event):
+		key = gtk.gdk.keyval_name(event.keyval)
+		# Match against capslock + control and control
+		if key in ( "f", "F") and event.state  in (gtk.gdk.CONTROL_MASK, gtk.gdk.LOCK_MASK|gtk.gdk.CONTROL_MASK) :
+			self.OnFind(widget)
+		elif key in ( "F3"):
+			self.OnFind(widget, repeat=True)
+
+	def OnFind(self, widget, repeat=False):
+		self.frame.OnFindTextview(None, widget, repeat=repeat)
+
+	def OnFindLogWindow(self, widget):
+		self.frame.OnFindTextview(None, self.RoomLog)
 		
 	def OnFindChatLog(self, widget):
-		self.frame.OnFindTextview(widget, self.ChatScroll)
+		self.frame.OnFindTextview(None, self.ChatScroll)
 		
 	def drag_data_get_data(self, treeview, context, selection, target_id, etime):
 		treeselection = treeview.get_selection()
@@ -971,14 +1013,22 @@ class ChatRoom:
 		self.popup_menu.set_user(user)
 		me = (self.popup_menu.user == None or self.popup_menu.user == self.frame.np.config.sections["server"]["login"])
 		
-		items[9].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-		items[10].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-		items[11].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-		items[12].set_active(self.frame.UserIpIsBlocked(user))
-		items[12].set_sensitive(not me)
-		items[13].set_active(self.frame.UserIpIsIgnored(user))
-		items[13].set_sensitive(not me) 
-		items[16].set_sensitive(not me)
+		#self.Menu_SendMessage
+		#self.Menu_ShowIPaddress
+		#self.Menu_GetUserInfo
+		#self.Menu_BrowseUser
+		#self.Menu_GivePrivileges
+
+		self.Menu_AddToList.set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+		self.Menu_BanUser.set_active(user in self.frame.np.config.sections["server"]["banlist"])
+		self.Menu_IgnoreUser.set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+		self.Menu_BlockUser.set_active(self.frame.UserIpIsBlocked(user))
+		self.Menu_BlockUser.set_sensitive(not me)
+		self.Menu_IgnoreIP.set_active(self.frame.UserIpIsIgnored(user))
+		self.Menu_IgnoreIP.set_sensitive(not me) 
+		#self.Menu_SearchUser
+		self.Menu_PrivateRooms.set_sensitive(not me)
+
 		self.popup_menu.editing = False	
 		self.popup_menu.popup(None, None, None, event.button, event.time)
 		
@@ -987,13 +1037,13 @@ class ChatRoom:
 	def OnShowChatHelp(self, widget):
 		self.frame.OnAboutChatroomCommands(widget, self.GetTabParent(self.Main))
 
-	def OnHideChatButtons(self, hide=False):
+	def OnShowChatButtons(self, show=True):
 		for widget in self.HideStatusLog, self.HideUserList, self.ShowChatHelp:
-			if hide:
-				widget.hide()
-			else:
+			if show:
 				widget.show()
-		if self.frame.np.config.sections["ui"]["speechenabled"] and not hide:
+			else:
+				widget.hide()
+		if self.frame.np.config.sections["ui"]["speechenabled"] and show:
 			self.Speech.show()
 		else:
 			self.Speech.hide()
@@ -1021,17 +1071,23 @@ class ChatRoom:
 	
 	def TickerSet(self, msg):
 		self.Ticker.set_ticker({})
-		for m in msg.msgs.keys():
-			self.Ticker.add_ticker(m, self.frame.np.decode(msg.msgs[m], self.encoding))
+		for user in msg.msgs.keys():
+			if user in self.frame.np.config.sections["server"]["ignorelist"] or self.frame.UserIpIsIgnored(user):
+				# User ignored, ignore Ticker messages
+				return
+			self.Ticker.add_ticker(user, self.frame.np.decode(msg.msgs[user], self.encoding))
 
 	def TickerAdd(self, msg):
-		text = self.frame.np.decode(msg.msg, self.encoding)
-		self.Ticker.add_ticker(msg.user, text)
+		user = msg.user
+		if user in self.frame.np.config.sections["server"]["ignorelist"] or self.frame.UserIpIsIgnored(user):
+			# User ignored, ignore Ticker messages
+			return
+		self.Ticker.add_ticker(msg.user, self.frame.np.decode(msg.msg, self.encoding))
 
 	def TickerRemove(self, msg):
 		self.Ticker.remove_ticker(msg.user)
 		
-	def SayChatRoom(self, msg, text):
+	def SayChatRoom(self, msg, text, public=False):
 		text = re.sub("\s\s+", "  ", text)
 		login = self.frame.np.config.sections["server"]["login"]
 		user = msg.user
@@ -1048,14 +1104,14 @@ class ChatRoom:
 				self.frame.ChatNotebook.request_hilite(self.Main)
 				if self.frame.ChatNotebook.is_tab_detached(self.Main):
 					if not self.frame.ChatNotebook.is_detached_tab_focused(self.Main):
-						self.frame.Notification("rooms", user, self.room)
+						self.frame.Notifications.Add("rooms", user, self.room, tab=False)
 				else:
-					self.frame.ChatRequestIcon(1)
+					self.frame.ChatRequestIcon(1, self.Main)
 						
 					# add hilite to trayicon
 					if self.frame.ChatNotebook.get_current_page() != self.frame.ChatNotebook.page_num(self.roomsctrl.joinedrooms[self.room].Main) or self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.hpaned1) or not self.frame.is_mapped:
 						if self.room not in self.frame.TrayApp.tray_status["hilites"]["rooms"]:
-							self.frame.Notification("rooms", user, self.room)
+							self.frame.Notifications.Add("rooms", user, self.room, tab=True)
 				#else:
 					#self.MainWindow.set_urgency_hint(False)
 			else:
@@ -1066,11 +1122,17 @@ class ChatRoom:
 					self.frame.ChatRequestIcon(0)
 
 		if text[:4] == "/me ":
-			line = "* %s %s" % (user, text[4:])
+			if public:
+				line = "%s | * %s %s" % (msg.room, user, text[4:])
+			else:
+				line = "* %s %s" % (user, text[4:])
 			speech = line[2:]
 			tag = self.tag_me
 		else:
-			line = "[%s] %s" % (user, text)
+			if public:
+				line = "%s | [%s] %s" % (msg.room, user, text)
+			else:
+				line = "[%s] %s" % (user, text)
 			speech = text
 
 		if len(self.lines) >= 400:
@@ -1088,7 +1150,7 @@ class ChatRoom:
 		if user != login:
 			self.lines.append(AppendLine(self.ChatScroll, self.frame.CensorChat(self.frame.np.decode(line, self.encoding)), tag, username=user, usertag=self.tag_users[user], timestamp_format=timestamp_format))
 			if self.Speech.get_active():
-				self.frame.new_tts(self.frame.np.config.sections["ui"]["speechrooms"] % {"room": self.room, "user": self.frame.tts_clean(user), "message": self.frame.tts_clean(self.frame.np.decode(speech, self.encoding))} )
+				self.frame.Notifications.new_tts(self.frame.np.config.sections["ui"]["speechrooms"] % {"room": self.room, "user": self.frame.Notifications.tts_clean(user), "message": self.frame.Notifications.tts_clean(self.frame.np.decode(speech, self.encoding))} )
 		else:
 			self.lines.append(AppendLine(self.ChatScroll, self.frame.np.decode(line, self.encoding), tag, username=user, usertag=self.tag_users[user], timestamp_format=timestamp_format))
 		
@@ -1244,9 +1306,9 @@ class ChatRoom:
 			self.frame.BothRescan()
 		elif cmd in ["/tick", "/t"]:
 			self.frame.np.queue.put(slskmessages.RoomTickerSet(self.room, self.frame.np.encode(args, self.encoding)))
-		elif cmd in ('/reload',):
-			self.frame.pluginhandler.reread()
-			self.frame.pluginhandler = pluginsystem.PluginHandler(self.frame)
+		#elif cmd in ('/reload',):
+			#self.frame.pluginhandler.reread()
+			#self.frame.pluginhandler = pluginsystem.PluginHandler(self.frame)
 		elif cmd[:1] == "/" and self.frame.pluginhandler.TriggerPublicCommandEvent(self.room, cmd[1:], args):
 			pass
 		elif cmd and cmd[:1] == "/" and cmd != "/me" and cmd[:2] != "//":
@@ -1284,8 +1346,8 @@ class ChatRoom:
 				self.clist.append(username)
 				if self.frame.np.config.sections["words"]["dropdown"]:
 					self.ChatEntry.get_completion().get_model().append([username])
-			
-		AppendLine(self.RoomLog, _("%s joined the room") % username, self.tag_log)
+		if username not in self.frame.np.config.sections["server"]["ignorelist"] and not self.frame.UserIpIsIgnored(username):
+			AppendLine(self.RoomLog, _("%s joined the room") % username, self.tag_log)
 		img = self.frame.GetStatusImage(userdata.status)
 		flag = userdata.country
 		if flag is not None:
@@ -1317,8 +1379,8 @@ class ChatRoom:
 							liststore.remove(iter)
 							break
 						iter = liststore.iter_next(iter)
-			
-		AppendLine(self.RoomLog, _("%s left the room") % username, self.tag_log)
+		if username not in self.frame.np.config.sections["server"]["ignorelist"] and not self.frame.UserIpIsIgnored(username):
+			AppendLine(self.RoomLog, _("%s left the room") % username, self.tag_log)
 		self.usersmodel.remove(self.users[username])
 		del self.users[username]
 		self.getUserTag(username)
@@ -1351,7 +1413,7 @@ class ChatRoom:
 	def UserColumnDraw(self, column, cellrenderer, model, iter):
 		user = self.usersmodel.get_value(iter, 2)
 		if self.room in self.roomsctrl.PrivateRooms:
-			if user == self.frame.np.config.sections["server"]["login"] and (self.roomsctrl.PrivateRooms[self.room]["owned"]):
+			if user == self.roomsctrl.PrivateRooms[self.room]["owner"]:
 				cellrenderer.set_property("underline", pango.UNDERLINE_SINGLE)
 				cellrenderer.set_property("weight", pango.WEIGHT_BOLD)
 			elif user in (self.roomsctrl.PrivateRooms[self.room]["operators"]):
@@ -1386,7 +1448,8 @@ class ChatRoom:
 			action = _("%s has gone away")
 		else:
 			action = _("%s has returned")
-		AppendLine(self.RoomLog, action % user, self.tag_log)
+		if user not in self.frame.np.config.sections["server"]["ignorelist"] and not self.frame.UserIpIsIgnored(user):
+			AppendLine(self.RoomLog, action % user, self.tag_log)
 		if user in self.tag_users.keys():
 			color = self.getUserStatusColor(status)
 			self.changecolour(self.tag_users[user], color)
@@ -1429,20 +1492,29 @@ class ChatRoom:
 	def UserNameEvent(self, tag, widget, event, iter, user):
 
 		if tag.last_event_type == gtk.gdk.BUTTON_PRESS and event.type == gtk.gdk.BUTTON_RELEASE and event.button in (1, 2):
-			items = self.popup_menu.get_children()
+			#items = self.popup_menu.get_children()
 			# Chat, Userlists use the normal popup system
 			self.popup_menu.editing = True
 			self.popup_menu.set_user(user)
 			me = (self.popup_menu.user == None or self.popup_menu.user == self.frame.np.config.sections["server"]["login"])
 			
-			items[9].set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
-			items[10].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-			items[11].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-			items[12].set_active(self.frame.UserIpIsBlocked(user))
-			items[12].set_sensitive(not me)
-			items[13].set_active(self.frame.UserIpIsIgnored(user))
-			items[13].set_sensitive(not me) 
-			items[16].set_sensitive(not me)
+
+			#self.Menu_SendMessage
+			#self.Menu_ShowIPaddress
+			#self.Menu_GetUserInfo
+			#self.Menu_BrowseUser
+			#self.Menu_GivePrivileges
+
+			self.Menu_AddToList.set_active(user in [i[0] for i in self.frame.np.config.sections["server"]["userlist"]])
+			self.Menu_BanUser.set_active(user in self.frame.np.config.sections["server"]["banlist"])
+			self.Menu_IgnoreUser.set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+			self.Menu_BlockUser.set_active(self.frame.UserIpIsBlocked(user))
+			self.Menu_BlockUser.set_sensitive(not me)
+			self.Menu_IgnoreIP.set_active(self.frame.UserIpIsIgnored(user))
+			self.Menu_IgnoreIP.set_sensitive(not me) 
+			#self.Menu_SearchUser
+			self.Menu_PrivateRooms.set_sensitive(not me)
+
 			self.popup_menu.editing = False
 			self.popup_menu.popup(None, None, None, event.button, event.time)
 		tag.last_event_type = event.type
@@ -1573,7 +1645,7 @@ class ChatRoom:
 			
 		for tag in self.tag_users.values():
 			self.changecolour(tag, "useroffline")
-			
+		self.Ticker.set_ticker({})
 	def Rejoined(self, users):
 		# Update user list with an inexpensive sorting function
 		self.usersmodel.set_default_sort_func(lambda *args: -1)
@@ -1617,7 +1689,7 @@ class ChatRoom:
 		else:
 			if not self.room in autojoin:
 				autojoin.append(self.room)
-		self.frame.np.config.writeConfig()
+		self.frame.np.config.writeConfiguration()
 		
 
 	def GetCompletionList(self, ix=0, text="", clist=[]):
@@ -1660,8 +1732,10 @@ class ChatRoom:
 			if new > max:
 				new = max
 			adj.set_value(new)
-		if event.keyval != gtk.gdk.keyval_from_name("Tab"):
-			self.midwaycompletion = False
+		# ISO_Left_Tab normally corresponds with shift+tab
+		if event.keyval not in (gtk.gdk.keyval_from_name("Tab"), gtk.gdk.keyval_from_name("ISO_Left_Tab")):
+			if event.keyval not in (gtk.gdk.keyval_from_name("Shift_L"), gtk.gdk.keyval_from_name("Shift_R")):
+				self.midwaycompletion = False
 			return False
 		config = self.frame.np.config.sections["words"]
 		if not config["tab"]:
@@ -1690,7 +1764,11 @@ class ChatRoom:
 				currentnick = self.completions['completions'][self.completions['currentindex']]
 			if self.midwaycompletion:
 				widget.delete_text(ix - len(currentnick), ix)
-				self.completions['currentindex'] = (self.completions['currentindex']+1) % len(self.completions['completions'])
+				direction = 1 # Forward cycle
+				if event.keyval == gtk.gdk.keyval_from_name("ISO_Left_Tab"):
+					direction = -1 # Backward cycle
+				self.completions['currentindex'] = (self.completions['currentindex'] + direction) % len(self.completions['completions'])
+
 				newnick = self.completions['completions'][self.completions['currentindex']]
 				widget.insert_text(newnick, ix)
 				widget.set_position(preix + len(newnick))
@@ -1698,10 +1776,16 @@ class ChatRoom:
 		return True
 
 	def OnLogToggled(self, widget):
-		if not widget.get_active() and self.logfile is not None:
-			self.logfile.close()
-			self.logfile = None
-	
+		if not widget.get_active():
+			if self.logfile is not None:
+				self.logfile.close()
+				self.logfile = None
+			if self.room in self.frame.np.config.sections["logging"]["rooms"]:
+				self.frame.np.config.sections["logging"]["rooms"].remove(self.room)
+		elif widget.get_active():
+			if self.room not in self.frame.np.config.sections["logging"]["rooms"]:
+				self.frame.np.config.sections["logging"]["rooms"].append(self.room)
+		
 	def OnEncodingChanged(self, widget):
 		try:
 			# PyGTK 2.6
@@ -1787,12 +1871,12 @@ class ChatRoom:
 						del config["ticker"]["rooms"][self.room]
 				else:
 					config["ticker"]["rooms"][self.room] = result
-				self.frame.np.config.writeConfig()
+				self.frame.np.config.writeConfiguration()
 			elif t == 2:
 				if self.room in config["ticker"]["rooms"]:
 					del config["ticker"]["rooms"][self.room]
 				config["ticker"]["default"] = result
-				self.frame.np.config.writeConfig()
+				self.frame.np.config.writeConfiguration()
 			self.frame.np.queue.put(slskmessages.RoomTickerSet(self.room, self.frame.np.encode(result, self.encoding)))
 		return True
 
