@@ -17,12 +17,13 @@
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
 import gtk
+from thread import start_new_thread
 
 from transferlist import TransferList
 from utils import PopupMenu, PressHeader
 from pynicotine import slskmessages
 import string, os
-from pynicotine.utils import _
+from pynicotine.utils import _, executeCommand
 from entrydialog import *
 
 class Downloads(TransferList):
@@ -140,7 +141,7 @@ class Downloads(TransferList):
 
 		user = model.get_value(iter, 0)
 		filename = model.get_value(iter, 1)
-		fullname = model.get_value(iter, 9)
+		fullname = model.get_value(iter, 10)
 		size = speed = "0"
 		length = bitrate = None
 		queue = immediate = num = country = bitratestr = ""
@@ -178,7 +179,7 @@ class Downloads(TransferList):
 
 		filemanager = filemanager_config.split()[0]
 		filemanager_args = filemanager_config.split(filemanager)[1]
-                arg = filemanager_args.split('$')[0].strip()
+		arg = filemanager_args.split('$')[0].strip()
 		complete_path = os.path.join(downloaddir, transfer.path)
 		arg_list = []
 		arg_list.append(filemanager)
@@ -254,27 +255,29 @@ class Downloads(TransferList):
 				self.OnAbortTransfer(widget, True, True)
 
 	def OnPlayFiles(self, widget, prefix = ""):
+		start_new_thread(self._OnPlayFiles, (widget, prefix))
+	def _OnPlayFiles(self, widget, prefix = ""):
 		executable = self.frame.np.config.sections["players"]["default"]
 		downloaddir = self.frame.np.config.sections["transfers"]["downloaddir"]
 		if "$" not in executable:
 			return
-		commandargs = executable.split(" ")
-		pos = commandargs.index("$")
 		for fn in self.selected_transfers:
 			if fn.file is None:
 				continue
-			command = commandargs
+			playfile = None
 			if os.path.exists(fn.file.name):
-				command[pos] = fn.file.name
+				playfile = fn.file.name
 			else:
+				"""
+If this file doesn't exist anymore, it may have finished downloading and have been
+renamed, try looking in the download directory and match the original filename.
+				"""
 				basename = string.split(fn.filename, '\\')[-1]
 				path = os.sep.join([downloaddir, basename])
 				if os.path.exists(path):
-					command[pos] = path
-			if command[pos] == "$":
-				continue
-			os.spawnlp(os.P_NOWAIT, command[0], *command)
-
+					playfile = path
+			if playfile:
+				executeCommand(executable, playfile, background=False)
 
 	def OnPopupMenuUsers(self, widget):
 		
@@ -329,10 +332,28 @@ class Downloads(TransferList):
 			items[i].set_sensitive(act)
 		return True
 
-					
+	def DoubleClick(self, event):
+		self.select_transfers()
+		dc = self.frame.np.config.sections["transfers"]["download_doubleclick"]
+		
+		if dc == 1: # Send to player
+			self.OnPlayFiles(None)
+		elif dc == 2: #File manager
+			self.OnOpenDirectory(None)
+		elif dc == 3: #Search
+			self.OnFileSearch(None)
+		elif dc == 4: #Abort
+			self.OnAbortTransfer(None, False)
+		elif dc == 5: #Clear
+			self.OnClearTransfer(None)
+		elif dc == 6: #Retry
+			self.OnRetryTransfer(None)
+			
 	def OnPopupMenu(self, widget, event, kind):
 		if kind == "mouse":
 			if event.button != 3:
+				if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+					self.DoubleClick(event)
 				return False
 		
 		self.selected_transfers = []
@@ -404,7 +425,7 @@ class Downloads(TransferList):
 		self.select_transfers()
 		for transfer in self.selected_transfers:
 			self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
-			self.frame.MainNotebook.set_current_page(4)
+			self.frame.ChangeMainPage(None, "search")
 			break
 		
 	def OnRetryTransfer(self, widget):
