@@ -34,7 +34,7 @@ class PrivateChats(IconNotebook):
 	
 	def __init__(self, frame):
 		ui = frame.np.config.sections["ui"]
-		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"], ui["tab_icons"], ui["tab_reorderable"])
+		IconNotebook.__init__(self, frame.images, ui["labelprivate"], ui["tabclosers"], ui["tab_icons"], ui["tab_reorderable"], ui["tab_status_icons"])
 		self.popup_enable()
 
 		self.frame = frame
@@ -53,7 +53,7 @@ class PrivateChats(IconNotebook):
 				gobject.idle_add(tab.ChatLine.grab_focus)
 				# Remove hilite if selected tab belongs to a user in the hilite list
 				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
-					self.frame.ClearNotification("private", tab.user)
+					self.frame.Notifications.Clear("private", tab.user)
 					
 	def ClearNotifications(self):
 		if self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox):
@@ -63,7 +63,7 @@ class PrivateChats(IconNotebook):
 			if tab.Main == page:
 				# Remove hilite
 				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
-					self.frame.ClearNotification("private", tab.user)
+					self.frame.Notifications.Clear("private", tab.user)
 					
 	def Focused(self, page, focused):
 		if not focused:
@@ -71,13 +71,17 @@ class PrivateChats(IconNotebook):
 		for user, tab in self.users.items():
 			if tab.Main == page:
 				if user in self.frame.TrayApp.tray_status["hilites"]["private"]:
-					self.frame.ClearNotification("private", tab.user)
+					self.frame.Notifications.Clear("private", tab.user)
 					
 	def GetUserStatus(self, msg):
 		if msg.user in self.users:
 			tab = self.users[msg.user]
 			status = [_("Offline"), _("Away"), _("Online")][msg.status]
-			self.set_text(tab.Main, "%s (%s)" % (msg.user[:15], status))
+			if not self.frame.np.config.sections["ui"]["tab_status_icons"]:
+				self.set_text(tab.Main, "%s (%s)" % (msg.user[:15], status))
+			else:
+				self.set_text(tab.Main, msg.user)
+			self.set_status_image(tab.Main, msg.status)
 			tab.GetUserStatus(msg.status)
 
 	def SendMessage(self, user, text = None, direction = None):
@@ -146,6 +150,21 @@ class PrivateChats(IconNotebook):
 	def ShowMessage(self, msg, text, status=None):
 		if msg.user in self.frame.np.config.sections["server"]["ignorelist"]:
 			return
+		if msg.user in self.frame.np.users and type(self.frame.np.users[msg.user].addr) is tuple:
+			ip, port = self.frame.np.users[msg.user].addr
+			if self.frame.np.ipIgnored(ip):
+				#print "ignored message from IP:", ip, msg.user
+				return
+		else:
+			self.frame.np.queue.put(slskmessages.GetPeerAddress(msg.user))
+			self.frame.np.PrivateMessageQueueAdd(msg, text)
+			return
+
+		user_text = self.frame.pluginhandler.IncomingPrivateChatEvent(msg.user, text)
+		if user_text == None:
+			#print "Pluginsystem made me silence."
+			return
+		(u, text) = user_text
 
 		self.SendMessage(msg.user, None)
 		chat = self.users[msg.user]
@@ -155,15 +174,15 @@ class PrivateChats(IconNotebook):
 			# Only show notifications if window is not focused, and don't
 			# highlight main window
 			if not self.is_detached_tab_focused(chat.Main):
-				self.frame.Notification("private", msg.user)
+				self.frame.Notifications.Add("private", msg.user)
 		else:
 			# If tab isn't detached
 			# Hilight main private chats Label 
-			self.frame.RequestIcon(self.frame.PrivateChatTabLabel)
+			self.frame.RequestIcon(self.frame.PrivateChatTabLabel, chat.Main)
 			# Show notifications if the private chats notebook isn't selected,
 			# the tab is not selected, or the main window isn't mapped
 			if self.get_current_page() != self.page_num(chat.Main) or self.frame.MainNotebook.get_current_page() != self.frame.MainNotebook.page_num(self.frame.privatevbox) or not self.frame.is_mapped:
-				self.frame.Notification("private", msg.user)
+				self.frame.Notifications.Add("private", msg.user)
 
 		# SEND CLIENT VERSION to user if the following string is sent
 		ctcpversion = 0
@@ -173,6 +192,7 @@ class PrivateChats(IconNotebook):
 		self.users[msg.user].ShowMessage(text, status, msg.timestamp)	
 		if ctcpversion and self.frame.np.config.sections["server"]["ctcpmsgs"] == 0:
 			self.SendMessage(msg.user, "Nicotine-Plus %s" % version)
+		self.frame.pluginhandler.IncomingPrivateChatNotification(msg.user, text)
 
 	def UpdateColours(self):
 		for chat in self.users.values():
@@ -181,7 +201,7 @@ class PrivateChats(IconNotebook):
 	def RemoveTab(self, tab):
 		self.remove_page(tab.Main)
 		if tab.user in self.frame.TrayApp.tray_status["hilites"]["private"]:
-			self.frame.ClearNotification("private", tab.user)
+			self.frame.Notifications.Clear("private", tab.user)
 		del self.users[tab.user]
 		if tab.user in self.frame.np.config.sections["privatechat"]["users"]:
 			self.frame.np.config.sections["privatechat"]["users"].remove(tab.user)
@@ -203,6 +223,7 @@ class PrivateChats(IconNotebook):
 			self.users[user].ConnClose()
 			tab = self.users[user]
 			self.set_text(tab.Main, "%s (%s)" % (user[:15], _("Offline")))
+			self.set_status_image(tab.Main, 0)
 			tab.GetUserStatus(0)
 			
 	def UpdateCompletions(self):
@@ -241,12 +262,12 @@ class PrivateChat:
 		self.PrivateChatTab.destroy()
 		self.wTree.signal_autoconnect(self)
 		#Get glade widgets' tooltips
-		self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
+		#self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
 		#if not self.frame.np.config.sections["ui"]["tooltips"]:
-		self.tooltips.disable()
-		self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
-		if not self.frame.np.config.sections["ui"]["tooltips"]:
-			self.tooltips.disable()
+		#self.tooltips.disable()
+		#self.tooltips = gtk.tooltips_data_get(self.PeerPrivateMessages)[0]
+		#if not self.frame.np.config.sections["ui"]["tooltips"]:
+		#	self.tooltips.disable()
 
 
 
@@ -454,13 +475,13 @@ class PrivateChat:
 		if self.frame.away and not self.autoreplied and autoreply:
 			self.SendMessage("[Auto-Message] %s" % autoreply)
 			self.autoreplied = 1
-		self.frame.new_tts(self.frame.np.config.sections["ui"]["speechprivate"] %{"user":self.frame.tts_clean(self.user), "message": self.frame.tts_clean(speech)} )
+		self.frame.Notifications.new_tts(self.frame.np.config.sections["ui"]["speechprivate"] %{"user":self.frame.Notifications.tts_clean(self.user), "message": self.frame.Notifications.tts_clean(speech)} )
 
 	def SendMessage(self, text):
-		tuple = self.frame.pluginhandler.OutgoingPrivateChatEvent(self.user, text)
-		if tuple == None:
+		user_text = self.frame.pluginhandler.OutgoingPrivateChatEvent(self.user, text)
+		if user_text == None:
 			return
-		(u, text) = tuple
+		(u, text) = user_text
 
 		my_username = self.frame.np.config.sections["server"]["login"]
 		if text[:4] == "/me ":
@@ -775,6 +796,7 @@ class PrivateChat:
 		statusword = [_("Offline"), _("Away"), _("Online")][status]
 		title = _("Nicotine+ Private Chat: %(user)s (%(status)s)") % {'user':self.user, 'status':statusword}
 		self.chats.set_detached_tab_title(self.Main, title)
+		
 
 	
 	def OnClose(self, widget):
