@@ -82,10 +82,10 @@ class WishList( gtk.Dialog):
 		self.mainVbox.pack_start(self.RemoveWishButton, False, False, 0)
 
 
-		self.ClearWishesButton = self.nicotine.CreateIconButton(gtk.STOCK_CLEAR, "stock", self.OnClearWishes, _("Clear"))
+		self.SelectAllWishesButton = self.nicotine.CreateIconButton(gtk.STOCK_DND_MULTIPLE, "stock", self.OnSelectAllWishes, _("Select all"))
 
 		
-		self.mainVbox.pack_start(self.ClearWishesButton, False, False, 0)
+		self.mainVbox.pack_start(self.SelectAllWishesButton, False, False, 0)
 
 		self.CloseButton = self.nicotine.CreateIconButton(gtk.STOCK_CLOSE, "stock", self.quit, _("Close"))
 		self.mainVbox.pack_end(self.CloseButton, False, False, 0)
@@ -140,18 +140,9 @@ class WishList( gtk.Dialog):
 			self.removeWish(wish)
 	
 
-	def OnClearWishes(self, widget):
-		self.wishes = {}
-		self.store.clear()
-		for number, search in self.nicotine.Searches.searches.items():
-			search[4] = 0
-			self.nicotine.Searches.searches[number] = search
-			if search[2] is not None:
-				search[2].RememberCheckButton.set_active(False)
-
-		self.nicotine.np.config.sections["server"]["autosearch"] = []
-		self.nicotine.np.config.writeConfig()
-		
+	def OnSelectAllWishes(self, widget):
+		self.WishlistView.get_selection().select_all()
+	
 	def quit(self, w=None, event=None):
 
 		self.hide()
@@ -237,9 +228,9 @@ class Searches(IconNotebook):
 	def OnClearSearchHistory(self):
 		self.frame.SearchEntry.set_text("")
 		self.frame.np.config.sections["searches"]["history"] = []
-		self.frame.np.config.writeConfig()
+		self.frame.np.config.writeConfiguration()
 		self.frame.SearchEntryCombo.get_model().clear()
-	
+		self.frame.SearchEntryCombo.append_text("")
 	def OnSearch(self):
 		text = self.frame.SearchEntry.get_text().strip()
 		
@@ -310,7 +301,7 @@ class Searches(IconNotebook):
 		items.insert(0, text)
 		# Clear old items
 		del items[15:]
-		self.frame.np.config.writeConfig()
+		self.frame.np.config.writeConfiguration()
 		# Repopulate the combo list
 		self.frame.SearchEntryCombo.get_model().clear()
 		templist = []
@@ -398,7 +389,7 @@ class Searches(IconNotebook):
 		search = self.searches[id]
 		if search[1] in self.frame.np.config.sections["server"]["autosearch"]:
 			self.frame.np.config.sections["server"]["autosearch"].remove(search[1])
-			self.frame.np.config.writeConfig()
+			self.frame.np.config.writeConfiguration()
 		search[4] = 0
 		self.WishListDialog.removeWish(search[1])
 		
@@ -425,7 +416,7 @@ class Searches(IconNotebook):
 		if i[1] in self.frame.np.config.sections["server"]["autosearch"]:
 			return
 		self.frame.np.config.sections["server"]["autosearch"].append(i[1])
-		self.frame.np.config.writeConfig()
+		self.frame.np.config.writeConfiguration()
 		i[4] = 1
 		self.WishListDialog.addWish(i[1])
 		
@@ -498,9 +489,9 @@ class Search:
 	def __init__(self, Searches, text, id, mode, remember):
 		self.Searches = Searches
 		self.frame = Searches.frame
-		self.tooltips = self.frame.tooltips
-		if not self.frame.np.config.sections["ui"]["tooltips"]:
-			self.tooltips.disable()
+		#self.tooltips = self.frame.tooltips
+		#if not self.frame.np.config.sections["ui"]["tooltips"]:
+		#	self.tooltips.disable()
 
 		self.wTree = gtk.glade.XML(os.path.join(os.path.dirname(os.path.realpath(__file__)), "search.glade" ), None, 'nicotine' ) 
 		widgets = self.wTree.get_widget_prefix("")
@@ -510,7 +501,7 @@ class Search:
 		self.SearchTab.remove(self.Main)
 		self.SearchTab.destroy()
 		self.wTree.signal_autoconnect(self)
-
+		self._first_results = True
 		self.FilterBitrate_List = gtk.ListStore(gobject.TYPE_STRING)
 		self.FilterBitrate.set_model(self.FilterBitrate_List)
 		self.FilterBitrate.set_text_column(0)
@@ -543,7 +534,7 @@ class Search:
 		self.filters = None
 		# num, user, filename, h_size, h_speed, h_queue, immediatedl, h_bitrate, length, self.get_flag(user, country), directory, bitrate, fullpath, country,  size, speed, queue, status]
 		self.COLUMN_TYPES = [int, str, str, str, str, str, str, str, str, gtk.gdk.Pixbuf, str,
-			int, str, str, gobject.TYPE_ULONG, int, int, int]
+			int, str, str, gobject.TYPE_UINT64, int, int, int]
 		self.resultsmodel = gtk.TreeStore(* self.COLUMN_TYPES )
 
 		
@@ -571,7 +562,12 @@ class Search:
 		self.FilterSize.connect("changed", self.OnFilterChanged)
 		self.FilterBitrate.connect("changed", self.OnFilterChanged)
 		self.FilterCountry.connect("changed", self.OnFilterChanged)
-		
+		self.FilterIn.child.connect("activate", self.OnRefilter)
+		self.FilterOut.child.connect("activate", self.OnRefilter)
+		self.FilterSize.child.connect("activate", self.OnRefilter)
+		self.FilterBitrate.child.connect("activate", self.OnRefilter)
+		self.FilterCountry.child.connect("activate", self.OnRefilter)
+
 		self.selected_results = []
 		self.selected_users = []
 
@@ -579,15 +575,15 @@ class Search:
 		self.ResultsList.set_property("show-expanders", False)
 		self.ResultsList.set_property("rules-hint", True)
 		cols = InitialiseColumns(self.ResultsList,
-			[_("Number"), 50, "text", self.CellDataFunc],
+			[_("Number"), 50, "number", self.CellDataFunc],
 			[_("User"), 100, "text", self.CellDataFunc],
 			[_("Filename"), 250, "text", self.CellDataFunc],
-			[_("Size"), 100, "text", self.CellDataFunc],
+			[_("Size"), 100, "number", self.CellDataFunc],
 			[_("Speed"), 90, "text", self.CellDataFunc],
-			[_("In queue"), 50, "text", self.CellDataFunc],
+			[_("In queue"), 50, "number", self.CellDataFunc],
 			[_("Immediate Download"), 20, "text", self.CellDataFunc],
 			[_("Bitrate"), 50, "text", self.CellDataFunc],
-			[_("Length"), 50, "text", self.CellDataFunc],
+			[_("Length"), 50, "number", self.CellDataFunc],
 			[_("Country"), 25, "pixbuf"],
 			[_("Directory"), 1000, "text", self.CellDataFunc],
 			
@@ -607,8 +603,8 @@ class Search:
 			#col.connect("clicked", self.OnResort, ix)
 			#for r in col.get_cell_renderers():
 				#r.set_fixed_height_from_font(1)
-		for r in self.col_num.get_cell_renderers() + self.col_size.get_cell_renderers() +  self.col_length.get_cell_renderers():
-			r.set_property("xalign",1)
+		#for r in self.col_num.get_cell_renderers() + self.col_size.get_cell_renderers() +  self.col_length.get_cell_renderers():
+			#r.set_property("xalign",1)
 
 		self.col_num.set_sort_column_id(0)
 		self.col_user.set_sort_column_id(1)
@@ -715,14 +711,14 @@ class Search:
 		if user in self.users:
 			return
 		if user not in self.Searches.users.keys():
-			if user in self.frame.np.users.keys():
+			if user in self.frame.np.users.keys() and self.frame.np.users[user].status is not None:
 				self.Searches.users[user] = self.frame.np.users[user].status
 			else:
 				self.Searches.users[user] = 0
 				self.frame.np.queue.put(slskmessages.AddUser(user))
 			if user == self.frame.np.config.sections["server"]["login"]:
 				self.Searches.users[user] = 1
-				
+		
 		self.users.append(user)
 		results = []
 
@@ -737,7 +733,7 @@ class Search:
 			bitrate = ""
 			length = ""
 			br = 0
-			if result[3] == "mp3" and len(result[4]) == 3:
+			if result[3] != "" and len(result[4]) == 3:
 				a = result[4]
 				if a[2] == 1:
 					bitrate = _(" (vbr)")
@@ -750,7 +746,11 @@ class Search:
 			
 			if self._more_results == 0 and len(self.resultsmodel) < self.frame.np.config.sections['searches']["max_displayed_results"]:
 				self._more_results = 1
-				gobject.timeout_add(self.WAIT_BEFORE_DISPLAYING, self._realaddresults)
+				if self._first_results:
+					gobject.timeout_add(0, self._realaddresults)
+					self._first_results = False
+				else:
+					gobject.timeout_add(self.WAIT_BEFORE_DISPLAYING, self._realaddresults)
 			return len(results)
 	
 	def _realaddresults(self):
@@ -812,21 +812,22 @@ class Search:
 
 			self.all_data.append(row)
 			if (displaycounter + returned < self.frame.np.config.sections['searches']["max_displayed_results"]) and (not self.filters or self.check_filter(row)):
-				encoded_row = [itercounter, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl,
-				               h_bitrate, length, self.get_flag(user, country), encode(directory, user), bitrate,
-				               encode(fullpath, user), country,  size, speed, queue, status]
-				#print user, status
-				if user in self.usersiters:
-					iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
-				else:
-					try:
+				encoded_row = [itercounter, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl, h_bitrate, length, self.get_flag(user, country), encode(directory, user), bitrate, encode(fullpath, user), country,  size, speed, queue, status]
+				try:
+					if user in self.usersiters:
+						iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
+					else:
 						iter = self.resultsmodel.append(None, encoded_row)
-					except Exception, e:
-						print "Search row error", e, encoded_row
-						print 
-						iter=None
-
-				path = self.resultsmodel.get_path(iter)
+				except Exception, e:
+					types=[]
+					for i in encoded_row:
+						types.append( type(i))
+					#print types
+					print "Search row error:", e, encoded_row
+					iter=None
+				path = None
+				if iter is not None:
+					path = self.resultsmodel.get_path(iter)
 				if path is not None:
 					if self.usersGroup.get_active() and self.ExpandButton.get_active():
 						self.ResultsList.expand_to_path(path)
@@ -834,6 +835,8 @@ class Search:
 			itercounter += 1
 			if itercounter > self.frame.np.config.sections['searches']["max_stored_results"]:
 				break
+		
+		self.CountVisibleResults()
 		return returned
 			
 	def updateStatus(self, user, status):
@@ -1001,14 +1004,33 @@ class Search:
 					self.usersiters[user] = self.resultsmodel.append(None, [0, user, "", "", h_speed, h_queue, immediatedl, "", "", self.get_flag(user, country), "", 0, "", country, 0, speed, queue, status])
 				encoded_row = [ix, user, encode(filename, user), h_size, h_speed, h_queue, immediatedl, h_bitrate, length, self.get_flag(user, country), encode(directory, user), bitrate, encode( fullpath, user), country,  size, speed, queue, status]
 
-				if user in self.usersiters:
-					iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
-				else:
-					iter = self.resultsmodel.append(None, encoded_row)
+				try:
+					if user in self.usersiters:
+						iter = self.resultsmodel.append(self.usersiters[user], encoded_row)
+					else:
+						iter = self.resultsmodel.append(None, encoded_row)
+				except Exception, e:
+					print "Filters: Search row error:", e
+					for i in encoded_row:
+						print i, type(i),
+					print
 				displaycounter += 1
 			if displaycounter >= self.frame.np.config.sections['searches']["max_displayed_results"]:
 				break
+		self.CountVisibleResults()
 		
+	def CountVisibleResults(self):
+		if self.usersGroup.get_active():
+			iter_count = 0
+			user_count = self.resultsmodel.iter_n_children(None)
+
+			for i in range(user_count):
+				iters = self.resultsmodel.iter_nth_child(None, i)
+				iter_count += self.resultsmodel.iter_n_children(iters)
+		
+			self.Counter.set_text("Results: %d/%d" %(iter_count ,len(self.all_data)) )
+		else:
+			self.Counter.set_text("Results: %d/%d" %(self.resultsmodel.iter_n_children(None) ,len(self.all_data)) )
 
 	def OnPopupMenuUsers(self, widget):
 		
@@ -1358,7 +1380,7 @@ class Search:
 	def PushHistory(self, widget, title):
 		text = widget.child.get_text()
 		if not text.strip():
-        		return None
+			return None
 		text = text.strip()
 		history = self.frame.np.config.sections["searches"][title]
 		self.frame.np.config.pushHistory(history, text, 5)
