@@ -17,11 +17,12 @@
 # Copyright (c) 2003-2004 Hyriand. All rights reserved.
 
 import gtk
+from thread import start_new_thread
 
 from transferlist import TransferList
 from utils import PopupMenu, PressHeader
 import string, os
-from pynicotine.utils import _
+from pynicotine.utils import _, executeCommand
 from pynicotine import slskmessages
 from entrydialog import OptionDialog
 
@@ -106,18 +107,15 @@ class Uploads(TransferList):
 		
 		command = ""
 		if os.path.exists(transfer.path):
-
-			command = filemanager.replace("$", "\"%s\"" % transfer.path)
+			executeCommand(filemanager, transfer.path)
 		else:
-			command = filemanager.replace("$", "\"%s\"" % incompletedir)
-
-		os.system("%s  &" % command)
+			executeCommand(filemanager, incompletedir)
 		
 	def OnFileSearch(self, widget):
 		self.select_transfers()
 		for transfer in self.selected_transfers:
 			self.frame.SearchEntry.set_text(transfer.filename.rsplit("\\", 1)[1])
-			self.frame.MainNotebook.set_current_page(4)
+			self.frame.ChangeMainPage(None, "search")
 			break
 		
 	def OnExpandUploads(self, widget):
@@ -180,7 +178,7 @@ class Uploads(TransferList):
 			if user in self.frame.np.transfers.getTransferringUsers():
 				continue
 			self.frame.np.ProcessRequestToPeer(user, slskmessages.UploadQueueNotification(None))
-			self.frame.np.transfers.pushFile(user, filename, path)
+			self.frame.np.transfers.pushFile(user, filename, path, transfer)
 		self.frame.np.transfers.checkUploadQueue()
 
 			
@@ -218,18 +216,16 @@ class Uploads(TransferList):
 				self.OnAbortTransfer(widget, False, True)
 
 	def OnPlayFiles(self, widget, prefix = ""):
+		start_new_thread(self._OnPlayFiles, (widget, prefix))
+	def _OnPlayFiles(self, widget, prefix = ""):
 		executable = self.frame.np.config.sections["players"]["default"]
 		if "$" not in executable:
 			return
-		commandargs = executable.split(" ")
-		pos = commandargs.index("$")
 		for fn in self.selected_transfers:
-			command = commandargs
 			file = fn.filename.replace("\\", os.sep)
 			if os.path.exists(file):
-				command[pos] = file
-				os.spawnlp(os.P_NOWAIT, command[0], *command)
-			
+				executeCommand(executable, file, background=False)
+	
 	def OnPopupMenuUsers(self, widget):
 		
 		self.selected_transfers = []
@@ -286,6 +282,8 @@ class Uploads(TransferList):
 	def OnPopupMenu(self, widget, event, kind):
 		if kind == "mouse":
 			if event.button != 3:
+				if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
+					self.DoubleClick(event)
 				return False
 		
 		self.selected_transfers = []
@@ -337,6 +335,21 @@ class Uploads(TransferList):
 		self.frame.np.transfers.checkUploadQueue()
 		self.update()
 
+	def DoubleClick(self, event):
+		self.select_transfers()
+		dc = self.frame.np.config.sections["transfers"]["upload_doubleclick"]
+		
+		if dc == 1: # Send to player
+			self.OnPlayFiles(None)
+		elif dc == 2: #File manager
+			self.OnOpenDirectory(None)
+		elif dc == 3: #Search
+			self.OnFileSearch(None)
+		elif dc == 4: #Abort
+			self.OnAbortTransfer(None, False)
+		elif dc == 5: #Clear
+			self.OnClearTransfer(None)
+			
 	def OnAbortTransfer(self, widget, remove = False, clear = False):
 		self.select_transfers()
 		TransferList.OnAbortTransfer(self, widget, remove, clear)
@@ -355,4 +368,4 @@ class Uploads(TransferList):
 		TransferList.OnClearFailed(self, widget)
 		self.frame.np.transfers.calcUploadQueueSizes()
 		self.frame.np.transfers.checkUploadQueue()
-		self.update()
\ No newline at end of file
+		self.update()
