@@ -19,7 +19,7 @@
 import gtk
 import gobject
 import time
-import sys
+import sys, os
 
 from pynicotine import slskmessages
 from utils import InitialiseColumns, PopupMenu, InputDialog, Humanize, HumanSpeed, PressHeader
@@ -29,15 +29,32 @@ from pynicotine.utils import _
 class UserList:
 	def __init__(self, frame):
 		self.frame = frame
+		self.wTree = gtk.glade.XML(os.path.join(os.path.dirname(os.path.realpath(__file__)), "buddylist.glade" ), None, 'nicotine' ) 
+		widgets = self.wTree.get_widget_prefix("")
+		for i in widgets:
+			name = gtk.glade.get_widget_name(i)
+			self.__dict__[name] = i
+			#print name, i
+		self.TempWindow.remove(self.userlistvbox)
+		self.TempWindow.destroy()
+		self.wTree.signal_autoconnect(self)
+
+		TARGETS = [('text/plain', 0, 1)]
+		self.UserList.enable_model_drag_source(gtk.gdk.BUTTON1_MASK, TARGETS, gtk.gdk.ACTION_COPY)
+		self.UserList.enable_model_drag_dest(TARGETS,  gtk.gdk.ACTION_COPY)
+		self.UserList.connect("drag_data_get", self.buddylist_drag_data_get_data)
+		self.UserList.connect("drag_data_received", self.DragUserToBuddylist)
+
 		self.userlist = []
 		
 		self.usersmodel = gtk.ListStore(gtk.gdk.Pixbuf, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN, gobject.TYPE_BOOLEAN, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_INT, gobject.TYPE_STRING)
-		self.cols = cols = InitialiseColumns(self.frame.UserList,
-			[_("Status"), 20, "pixbuf"],
+		statusiconwidth = self.frame.images["offline"].get_width()+4
+		self.cols = cols = InitialiseColumns(self.UserList,
+			[_("Status"), statusiconwidth, "pixbuf"],
 			[_("Country"), 25, "pixbuf"],
 			[_("User"), 120, "text", self.CellDataFunc],
-			[_("Speed"), 0, "text", self.CellDataFunc],
-			[_("Files"), 0, "text", self.CellDataFunc],
+			[_("Speed"), 0, "number", self.CellDataFunc],
+			[_("Files"), 0, "number", self.CellDataFunc],
 			[_("Trusted"), 0, "toggle"],
 			[_("Notify"), 0, "toggle"],
 			[_("Privileged"), 0, "toggle"],
@@ -68,16 +85,16 @@ class UserList:
 			cols[1].set_visible(0)
 			config["columns"]["userlist"][1] = 0
 		for render in self.col_trusted.get_cell_renderers():
-			render.connect('toggled', self.cell_toggle_callback, self.frame.UserList, 5)
+			render.connect('toggled', self.cell_toggle_callback, self.UserList, 5)
 		for render in self.col_notify.get_cell_renderers():
-			render.connect('toggled', self.cell_toggle_callback, self.frame.UserList, 6)
+			render.connect('toggled', self.cell_toggle_callback, self.UserList, 6)
 		for render in self.col_privileged.get_cell_renderers():
-			render.connect('toggled', self.cell_toggle_callback, self.frame.UserList, 7)
+			render.connect('toggled', self.cell_toggle_callback, self.UserList, 7)
 		renderers = self.col_comments.get_cell_renderers()
 		for render in renderers:
-			render.connect('edited', self.cell_edited_callback, self.frame.UserList, 9)
-		self.frame.UserList.set_model(self.usersmodel)
-		self.frame.UserList.set_property("rules-hint", True)
+			render.connect('edited', self.cell_edited_callback, self.UserList, 9)
+		self.UserList.set_model(self.usersmodel)
+		self.UserList.set_property("rules-hint", True)
 		self.privileged = []
 		self.notify = []
 		self.trusted = []
@@ -120,6 +137,7 @@ class UserList:
 			iter = self.usersmodel.append(row)
 			self.userlist.append([user[0], user[1], last_seen, iter, flag])
 		self.usersmodel.set_sort_column_id(2, gtk.SORT_ASCENDING)
+		self.Popup_Menu_PrivateRooms = PopupMenu(self.frame)
 		self.popup_menu = popup = PopupMenu(frame)
 		popup.setup(
 			("#" + _("Send _message"), popup.OnSendMessage, gtk.STOCK_EDIT),
@@ -137,9 +155,66 @@ class UserList:
 			("", None),
 			("#" + _("Edit _comments"), self.OnEditComments, gtk.STOCK_EDIT),
 			("#" + _("_Remove"), self.OnRemoveUser, gtk.STOCK_CANCEL),
+			(1, _("Private rooms"), self.Popup_Menu_PrivateRooms, popup.OnPrivateRooms),
 		)
-		self.frame.UserList.connect("button_press_event", self.OnPopupMenu)
+		items = self.popup_menu.get_children()
+		self.Menu_SendMessage = items[0]
+		self.Menu_ShowIPaddress = items[2]
+		self.Menu_GetUserInfo = items[3]
+		self.Menu_BrowseUser = items[4]
+		self.Menu_GivePrivileges = items[5]
+		self.Menu_BanUser = items[6]
+		self.Menu_IgnoreUser = items[7]
+		self.Menu_OnNotify = items[9]
+		self.Menu_OnPrivileged = items[10]
+		self.Menu_Trusted = items[11]
+		self.Menu_EditComments = items[13]
+		self.Menu_RemoveUser = items[14]
+		self.Menu_PrivateRooms = items[15]
+
+		self.UserList.connect("button_press_event", self.OnPopupMenu)
+
+	def OnMoveList(self, widget):
+		tab = always = chatrooms = False
+		if self.frame.buddylist_in_tab.get_active():
+			tab = True
+		if self.frame.buddylist_always_visible.get_active():
+			always = True
+		if self.frame.buddylist_in_chatrooms1.get_active():
+			chatrooms = True
+		if tab:
+			self.frame.buddylist_in_chatrooms1.set_active(True)
+			self.frame.OnChatRooms(None)
+		if always:
+			self.frame.buddylist_in_tab.set_active(True)
+		if chatrooms:
+			self.frame.buddylist_always_visible.set_active(True)
+
+	def OnAddUser(self, widget):
+		text = widget.get_text()
+		if not text:
+			return
+		widget.set_text("")
+		self.AddToList(text)
+
+	def UpdateColours(self):
+		self.frame.SetTextBG(self.AddUserEntry)
+	
+	def buddylist_drag_data_get_data(self, treeview, context, selection, target_id, etime):
+		treeselection = treeview.get_selection()
+		model, iter = treeselection.get_selected()
+		status, flag, user, speed, files, trusted, notify, privileged, lastseen, comments = model.get(iter, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9)
+		#data = (status, flag, user, speed, files, trusted, notify, privileged, lastseen, comments)
+		selection.set(selection.target, 8, user)
 		
+	def DragUserToBuddylist(self, treeview, context, x, y, selection, info, etime):
+		model = treeview.get_model()
+		user = selection.data
+		if user:
+			self.AddToList(user)
+	def OnSettingsBanIgnore(self, widget):
+		self.frame.OnSettingsBanIgnore(widget)
+
 	def CellDataFunc(self, column, cellrenderer, model, iter):
 		colour = self.frame.np.config.sections["ui"]["search"]
 		if colour == "":
@@ -224,11 +299,11 @@ class UserList:
 	
 	def OnPopupMenu(self, widget, event):
 		items = self.popup_menu.get_children()
-		d = self.frame.UserList.get_path_at_pos(int(event.x), int(event.y))
+		d = self.UserList.get_path_at_pos(int(event.x), int(event.y))
 
 		if d:
 			path, column, x, y = d
-			user = self.frame.UserList.get_model().get_value(self.frame.UserList.get_model().get_iter(path), 2)
+			user = self.UserList.get_model().get_value(self.UserList.get_model().get_iter(path), 2)
 			
 			if event.button != 3:
 				if event.type == gtk.gdk._2BUTTON_PRESS:
@@ -239,13 +314,22 @@ class UserList:
 			self.popup_menu.set_user(user)
 			
 			items = self.popup_menu.get_children()
+			me = (self.popup_menu.user == None or self.popup_menu.user == self.frame.np.config.sections["server"]["login"])
 			
-			items[6].set_active(user in self.frame.np.config.sections["server"]["banlist"])
-			items[7].set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
-			items[9].set_active(user in self.notify)
-			items[10].set_active(user in self.privileged)
-			items[11].set_active(user in self.trusted)
-			
+			#self.Menu_SendMessage
+			#self.Menu_ShowIPaddress
+			#self.Menu_GetUserInfo
+			#self.Menu_BrowseUser
+			#self.Menu_GivePrivileges
+			self.Menu_BanUser.set_active(user in self.frame.np.config.sections["server"]["banlist"])
+			self.Menu_IgnoreUser.set_active(user in self.frame.np.config.sections["server"]["ignorelist"])
+			self.Menu_OnNotify.set_active(user in self.notify)
+			self.Menu_OnPrivileged.set_active(user in self.privileged)
+			self.Menu_Trusted.set_active(user in self.trusted)
+			#self.Menu_EditComments
+			#self.Menu_RemoveUser
+			self.Menu_PrivateRooms.set_sensitive(not me) # Private rooms
+
 			self.popup_menu.popup(None, None, None, event.button, event.time)
 		
 	def GetIter(self, user):
@@ -346,11 +430,11 @@ class UserList:
 			user, comment, seen, iter, flag = i
 			l.append([user, comment, (user in self.notify), (user in self.privileged), (user in self.trusted), seen, flag])
 		self.frame.np.config.sections["server"]["userlist"] = l
-		self.frame.np.config.writeConfig()
+		self.frame.np.config.writeConfiguration()
 		
 	def saveColumns(self):
 		columns = []
-		for column in self.frame.UserList.get_columns():
+		for column in self.UserList.get_columns():
 			columns.append(column.get_visible())
 		self.frame.np.config.sections["columns"]["userlist"] = columns
 		
