@@ -124,6 +124,10 @@ def InitialiseColumns(treeview, *args):
 		if c[2] == "text":
 			renderer = gtk.CellRendererText()
 			column = gtk.TreeViewColumn(c[0], renderer, text = i)
+		elif c[2] == "number":
+			renderer = gtk.CellRendererText()
+			column = gtk.TreeViewColumn(c[0], renderer, text = i)
+			renderer.set_property("xalign", 1)
 		elif c[2] == "colored":
 			renderer = gtk.CellRendererText()
 			column = gtk.TreeViewColumn(c[0], renderer, text = i, foreground = c[3][0], background = c[3][1])
@@ -142,6 +146,7 @@ def InitialiseColumns(treeview, *args):
 		elif c[2] == "toggle":
 			renderer = gtk.CellRendererToggle()
 			column = gtk.TreeViewColumn(c[0], renderer, active = i)
+			renderer.set_property("xalign", 1)
 		else:
 			renderer = gtk.CellRendererPixbuf()
 			column = gtk.TreeViewColumn(c[0], renderer, pixbuf = i)
@@ -208,36 +213,46 @@ def UrlEvent(tag, widget, event, iter, url):
 	if tag.last_event_type == gtk.gdk.BUTTON_PRESS and event.type == gtk.gdk.BUTTON_RELEASE and event.button == 1:
 		if url[:4] == "www.":
 			url = "http://" + url
-		protocol = url[:url.find(":")]
-		if protocol in PROTOCOL_HANDLERS:
-			if NICOTINE.browser is not None and NICOTINE.np.config.sections["ui"]["open_in_mozembed"] and protocol == "http":
-				NICOTINE.browser.load_url(url, 0)
-				tag.last_event_type = event.type
-				return
-			if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
-				PROTOCOL_HANDLERS[protocol](url.strip())
-			elif PROTOCOL_HANDLERS[protocol]:
-				executeCommand(PROTOCOL_HANDLERS[protocol], url)
-			else:
-				webbrowser.open(url)
-		else:
-			try:
-				import gnomevfs
-			except Exception, e:
-				try:
-					import gnome.vfs
-				except:
-					pass
-				else:
-					gnome.url_show(url)
-			else:
-				try:
-					gnomevfs.url_show(url)
-				except:
-					pass
-				
+		OpenUri(url)
 	tag.last_event_type = event.type
 
+def OpenUri(uri):
+	"""Open a URI in an external (web) browser. The given argument has
+	to be a properly formed URI including the scheme (fe. HTTP).
+	
+	As of now failures will be silently discarded."""
+	# Situation 1, user defined a way of handling the protocol
+	protocol = uri[:uri.find(":")]
+	if protocol in PROTOCOL_HANDLERS:
+		if NICOTINE.browser is not None and NICOTINE.np.config.sections["ui"]["open_in_mozembed"] and protocol in ("http", 'https'):
+			NICOTINE.browser.load_url(uri, 0)
+			return
+		if PROTOCOL_HANDLERS[protocol].__class__ is types.MethodType:
+			PROTOCOL_HANDLERS[protocol](uri.strip())
+			return
+		if PROTOCOL_HANDLERS[protocol]:
+			executeCommand(PROTOCOL_HANDLERS[protocol], uri)
+			return
+	# Situation 2, user did not define a way of handling the protocol, we'll leave it up to python
+	if webbrowser:
+		webbrowser.open(uri)
+		return
+	# Situation 3a, we let Gnome (new way?) deal with it
+	try:
+		import gnomevfs
+		gnomevfs.url_show(uri)
+		return
+	except Exception, e:
+		pass
+	# Situation 3b, we let Gnome (old way?) deal with it
+	try:
+		# import gnome.vfs <-- doesn't make sense, vfs was never used!
+		import gnome
+		gnome.url_show(uri)
+		return
+	except:
+		pass
+
 def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, timestamp_format = "%H:%M:%S", username=None, usertag=None, scroll=True):
 	line = str(line) # Error messages are sometimes tuples
 	def _makeurltag(buffer, tag, url):
@@ -258,6 +273,23 @@ def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, tim
 		else:
 			buffer.insert(iter, text)
 
+	def _usertag(buffer, section):
+		# Tag usernames with popup menu creating tag, and away/online/offline colors
+		if USERNAMEHOTSPOTS and username != None and usertag != None:
+			np = re.compile(re.escape(username))
+			match = np.search(section)
+			if match != None:
+				start2 = section[:match.start()]
+				name = match.group()[:]
+				start = section[match.end():]
+				_append(buffer, start2, tag)
+				_append(buffer, name, usertag)
+				_append(buffer, start, tag)
+			else:
+				_append(buffer, section, tag)
+		else:
+			_append(buffer, section, tag)
+
 	scrolledwindow = textview.get_parent()
 	va = scrolledwindow.get_vadjustment()
 	bottom = va.value >= (va.upper - int(va.page_size*1.5))
@@ -268,60 +300,44 @@ def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, tim
 
 	if line.startswith("* "):
 		ME = 1
+	TIMESTAMP = None
+	TS = 0 
 	if NICOTINE.np.config.sections["logging"]["timestamps"] and showstamp:
 		if timestamp_format and not timestamp:
-			line = "%s %s\n" % (recode(time.strftime(timestamp_format)), line)
+			TIMESTAMP = recode(time.strftime(timestamp_format))
+			line = "%s %s\n" % (TIMESTAMP, line)
 		elif timestamp_format and timestamp:
-			line = "%s %s\n" % (recode(time.strftime(timestamp_format, time.localtime(timestamp))), line)
+			TIMESTAMP = recode(time.strftime(timestamp_format, time.localtime(timestamp)))
+			line = "%s %s\n" % (TIMESTAMP, line)
 	else:
 		line += "\n"
-		
+	if TIMESTAMP is not None:
+		TS = len(TIMESTAMP)
+	# Append timestamp, if one exists, cut it from remaining line (to avoid matching against username)
+	_append(buffer, line[:TS], tag)
+	line = line[TS:]
+	# Match first url
 	match = URL_RE.search(line)
 	# Highlight urls, if found and tag them
 	while CATCH_URLS and match:
 		start = line[:match.start()]
+		_usertag(buffer, start)
 		url = match.group()[:-1]
 		urltag = _makeurltag(buffer, tag, url)
 		line = line[match.end()-1:]
 		
-		if USERNAMEHOTSPOTS and username != None and usertag != None and not ME:
-			np = re.compile(re.escape(username))
-			match = np.search(start)
-			if match != None:
-				start2 = start[:match.start()]
-				name = match.group()[:]
-				start = start[match.end():]
-				_append(buffer, start2, tag)
-				_append(buffer, name, usertag)
-				_append(buffer, start, tag)
-			else:
-				_append(buffer, start, tag)
-		else:
-			_append(buffer, start, tag)
+
 		if url.startswith("slsk://") and HUMANIZE_URLS:
 			
 			url = urllib.url2pathname( url)
 
 		_append(buffer, url, urltag)
+		# Match remaining url
 		match = URL_RE.search(line)
 	
 	
 	if line:
-		
-		if USERNAMEHOTSPOTS and username != None and usertag != None and not ME:
-			np = re.compile(re.escape(username))
-			match = np.search(line)
-			if match != None:
-				start = line[:match.start()]
-				name = match.group()[:]
-				line = line[match.end():]
-				_append(buffer, start, tag)
-				_append(buffer, name, usertag)
-				_append(buffer, line, tag)
-			else:
-				_append(buffer, line, tag)
-		else:
-			_append(buffer, line, tag)
+		_usertag(buffer, line)
 	
 	if scroll and bottom:
 		gobject.idle_add(ScrollBottom, scrolledwindow)
@@ -329,19 +345,21 @@ def AppendLine(textview, line, tag = None, timestamp = None, showstamp=True, tim
 	return linenr
 	
 class ImageLabel(gtk.HBox):
-	def __init__(self, label = "", image = None, onclose = None, closebutton = False, angle = 0, show_image = True):
+	def __init__(self, label = "", image = None, onclose = None, closebutton = False, angle = 0, show_image = True, statusimage=None, show_status_image=False):
 		gtk.HBox.__init__(self)
 		
 		self.closebutton = closebutton
 		self.angle = angle
 		self._show_image = show_image
+		self._show_status_image = show_status_image
 		self.notify = 0
 		
 		self._entered = 0
 		self._pressed = 0
 		
 		self.onclose = onclose
-		
+		self.status_img = None
+		self.statusimage = gtk.Image()
 		self.label = gtk.Label()
 		self.text = label
 		if NICOTINE.np.config.sections["ui"]["tab_colors"]:
@@ -351,18 +369,22 @@ class ImageLabel(gtk.HBox):
 		if not color:
 			self.label.set_text("%s" % self.text)
 		else:
-			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;")))
+			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;")))
 			
 		self.label.set_alignment(0.0, 0.50)
 		self.label.set_angle(angle)
 		self.label.show()
 
+		if self._show_status_image:
+			self.set_status_image(statusimage)
+			self.statusimage.show()
+
 		self.image = gtk.Image()
 		self.set_image(image)
 		
 		if self._show_image:
 			self.image.show()
-
+			
 
 		self._pack_children()
 		self._order_children()
@@ -385,7 +407,8 @@ class ImageLabel(gtk.HBox):
 		self.Box.set_spacing(2)
 		self.add(self.Box)
 		self.Box.show()
-		
+
+		self.Box.pack_start(self.statusimage, False, False)
 		self.Box.pack_start(self.label, True, True)
 		self.Box.pack_start(self.image, False, False)
 		if self.closebutton and self.onclose is not None:
@@ -398,14 +421,18 @@ class ImageLabel(gtk.HBox):
 				self.Box.reorder_child(self.button, 0)
 				self.Box.reorder_child(self.image, 1)
 				self.Box.reorder_child(self.label, 2)
+				self.Box.reorder_child(self.statusimage, 3)
 			else:
 				self.Box.reorder_child(self.image, 0)
 				self.Box.reorder_child(self.label, 1)
+				self.Box.reorder_child(self.statusimage, 2)
 		else:
-			self.Box.reorder_child(self.label, 0)
-			self.Box.reorder_child(self.image, 1)
+			self.Box.reorder_child(self.statusimage, 0)
+			self.Box.reorder_child(self.label, 1)
+			self.Box.reorder_child(self.image, 2)
 			if "button" in self.__dict__ and self.closebutton != 0:
-				self.Box.reorder_child(self.button, 2)
+				self.Box.reorder_child(self.button, 3)
+				
 
 				
 	def set_onclose(self, closebutton):
@@ -480,15 +507,29 @@ class ImageLabel(gtk.HBox):
 			self.label.set_text("%s" % self.text)
 			
 		else:
-			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;")))
+			self.label.set_markup("<span foreground=\"%s\">%s</span>" % (color, self.text.replace("<", "&lt;").replace(">", "&gt;").replace("&", "&amp;")))
 			
 	def set_image(self, img):
 		self.img = img
 		self.image.set_from_pixbuf(img)
-	
+
 	def get_image(self):
 		return self.img
 		
+	def set_status_image(self, img):
+		if img is self.status_img:
+			return
+		if NICOTINE:
+			if NICOTINE.np.config.sections["ui"]["tab_status_icons"]:
+				self.statusimage.show()
+			else:
+				self.statusimage.hide()
+		self.status_img = img
+		self.statusimage.set_from_pixbuf(img)
+
+	def get_status_image(self):
+		return self.status_img
+
 	def set_text(self, lbl):
 		if NICOTINE.np.config.sections["ui"]["tab_colors"]:
 			self.set_text_color( notify = None, text = lbl)
@@ -500,12 +541,13 @@ class ImageLabel(gtk.HBox):
 		return self.label.get_text()
 	
 class IconNotebook(gtk.Notebook):
-	def __init__(self, images, angle = 0, tabclosers = False, show_image = True, reorderable = True):
+	def __init__(self, images, angle = 0, tabclosers = False, show_image = True, reorderable = True, show_status_image = False):
 		self.tabclosers = tabclosers
 		self.reorderable = reorderable
 		gtk.Notebook.__init__(self)
 		self.images = images
 		self._show_image = show_image
+		self._show_status_image = show_status_image
 		self.pages = []
 		self.detached_tabs = []
 		self.connect("switch-page", self.dismiss_icon)
@@ -559,7 +601,7 @@ class IconNotebook(gtk.Notebook):
 		self.set_tab_angle(angle)
 		closebutton = self.tabclosers
 
-		label_tab = ImageLabel(label, self.images["empty"], onclose, closebutton = closebutton, angle = angle, show_image = self._show_image)
+		label_tab = ImageLabel(label, self.images["empty"], onclose, closebutton = closebutton, angle = angle, show_image = self._show_image, statusimage=None, show_status_image=self._show_status_image)
 		# menu for all tabs
 		label_tab_menu = ImageLabel(label, self.images["empty"])
 		self.pages.append([page, label_tab, 0, label_tab_menu])
@@ -618,21 +660,23 @@ class IconNotebook(gtk.Notebook):
 		window.show()
 		
 	def OnFocusIn(self, widget, event):
-		widget.set_icon(NICOTINE.images["n"])
+		
 		for item in self.detached_tabs:
 			if item[2] == widget:
 				item[3] = True
-				self.Focused(item[0], True)
+				self.OnFocused(item)
 				
 	def OnFocusOut(self, widget, event):
 		for item in self.detached_tabs:
 			if item[2] == widget:
 				item[3] = False
-				self.Focused(item[0], False)
+				#self.OnFocused(item)
+
+	def OnFocused(self, item):
+		(page, label, window, focused) = item
+		self.frame.Notifications.ClearPage(self, item)
+		self.set_detached_icon(page, 0)
 
-	def Focused(self, page, focused):
-		pass
-	
 	def attach_tab(self, page, destroying=False):
 		pagewidget = label_tab = label_tab_menu = label = status = None
 
@@ -683,7 +727,7 @@ class IconNotebook(gtk.Notebook):
 		return False
 	
 	def set_detached_icon(self, page, status):
-		image = self.images[("empty", "online", "hilite")[status]]
+		image = self.images[("n", "hilite3", "hilite")[status]]
 		for item in self.detached_tabs:
 			if item[0] is page:
 				window = item[2]
@@ -697,9 +741,17 @@ class IconNotebook(gtk.Notebook):
 				
 	def on_tab_click(self, widget, event, child):
 		pass
-	
+
+	def set_status_image(self, page, status):
+		image = self.images[("offline", "away", "online")[status]]
+		for i in self.pages:
+			if page == i[0]:
+				i[1].set_status_image(image)
+				i[3].set_status_image(image)
+				return
+
 	def set_image(self, page, status):
-		image = self.images[("empty", "online", "hilite")[status]]
+		image = self.images[("empty", "hilite3", "hilite")[status]]
 		for i in self.pages:
 			if page == i[0]:
 				if status == 1 and i[2] == 2:
@@ -735,9 +787,10 @@ class IconNotebook(gtk.Notebook):
 
 	def request_hilite(self, page):
 		if self.is_tab_detached(page):
-			if self.is_detached_tab_focused(page):
-				return
-			self.set_detached_icon(page, 2)
+			if not self.is_detached_tab_focused(page):
+				self.set_detached_icon(page, 2)
+			return
+			# Don't set 'tab' notification icons for detached tabs
 		current = self.get_nth_page(self.get_current_page())
 		if current == page:
 			return
@@ -746,9 +799,10 @@ class IconNotebook(gtk.Notebook):
 
 	def request_changed(self, page):
 		if self.is_tab_detached(page):
-			if self.is_detached_tab_focused(page):
-				return
-			self.set_detached_icon(page, 1)
+			if not self.is_detached_tab_focused(page):
+				self.set_detached_icon(page, 1)
+			return
+			# Don't set 'tab' notification icons for detached tabs
 		current = self.get_nth_page(self.get_current_page())
 		if current == page:
 			return
@@ -940,6 +994,31 @@ class PopupMenu(gtk.Menu):
 			except Exception, e:
 				print e
 
+	def OnPrivateRooms(self, widget):
+		if self.user == None or self.user == self.frame.np.config.sections["server"]["login"]:
+			return False
+		user = self.user
+		items = []
+		popup = self.frame.userlist.Popup_Menu_PrivateRooms
+		popup.clear()
+		popup.set_user(self.user)
+		#print self.roomsctrl.PrivateRooms
+		for room in self.frame.chatrooms.roomsctrl.PrivateRooms:
+			if not (self.frame.chatrooms.roomsctrl.IsPrivateRoomOwned(room) or self.frame.chatrooms.roomsctrl.IsPrivateRoomOperator(room)):
+				continue
+			if self.user in self.frame.chatrooms.roomsctrl.PrivateRooms[room]["users"]:
+				items.append(("#" + _("Remove from private room %s" %room), popup.OnPrivateRoomRemoveUser, gtk.STOCK_REMOVE, room))
+			else:
+				items.append(("#" + _("Add to private room %s" %room), popup.OnPrivateRoomAddUser, gtk.STOCK_ADD, room))
+			if self.frame.chatrooms.roomsctrl.IsPrivateRoomOwned(room):
+				if self.user in self.frame.chatrooms.roomsctrl.PrivateRooms[room]["operators"]:
+					items.append(("#" + _("Remove as operator of %s" %room), popup.OnPrivateRoomRemoveOperator, gtk.STOCK_REMOVE, room))
+				else:
+					items.append(("#" + _("Add as operator of %s" %room), popup.OnPrivateRoomAddOperator, gtk.STOCK_ADD, room))
+
+		popup.setup(*items)
+		
+		return True
 
 def InputDialog(parent, title, message, default = ""):
 	dlg = gtk.Dialog(title = title, parent = parent,
@@ -1268,7 +1347,7 @@ def SaveEncoding(np, section, entry, encoding):
 		np.config.sections["server"][section][entry] = encoding
 	elif entry in np.config.sections["server"][section]:
 		del np.config.sections["server"][section][entry]
-	np.config.writeConfig()
+	np.config.writeConfiguration()
 	
 class ImportWinSlskConfig:
 	def __init__(self, config, Path, Queue, Login, Rooms, BuddyList, BanList, IgnoreList, UserInfo, UserImage):
