@@ -22,8 +22,9 @@
 import struct
 import types
 import zlib
-import os
+import os, sys
 from utils import _
+from utils import *
 from logfacility import log
 
 """ This module contains message classes, that networking and UI thread
@@ -34,9 +35,20 @@ Id = 99
 
 def newId():
 	global Id
-	Id = Id + 1
+	Id += 1
 	return Id
 
+# New object type to fix Ticket #486, "Search don't work"
+class NetworkIntType(object):
+	def __init__(self, value):
+		self.value = value
+class NetworkLongType(object):
+	def __init__(self, value):
+		self.value = value
+class NetworkLongLongType(object):
+	def __init__(self, value):
+		self.value = value
+
 class InternalMessage:
 	pass
 
@@ -121,6 +133,11 @@ class SetUploadLimit(InternalMessage):
 		self.limit = limit
 		self.limitby = limitby
 
+class SetDownloadLimit(InternalMessage):
+	""" Sent by the GUI thread to indicate changes in bandwidth shaping rules"""
+	def __init__(self, limit):
+		self.limit = limit
+
 class SetGeoBlock(InternalMessage):
 	""" Sent by the GUI thread to indicate changes in GeoIP blocking"""
 	def __init__(self, config):
@@ -166,7 +183,6 @@ class DebugMessage(InternalMessage):
 
 class SlskMessage:
 	""" This is a parent class for all protocol messages. """
-	
 	def getObject(self, message, type, start=0, getintasshort=0, getsignedint=0):
 		""" Returns object of specified type, extracted from message (which is
 		a binary array). start is an offset."""
@@ -180,7 +196,10 @@ class SlskMessage:
 				else:
 					return intsize+start, struct.unpack("<I", message[start:start+intsize])[0]
 			elif type is types.LongType:
-				return struct.calcsize("<L")+start, struct.unpack("<L", message[start:start+struct.calcsize("<L")])[0]
+				if getsignedint:
+					return struct.calcsize("<Q")+start, struct.unpack("<Q", message[start:start+struct.calcsize("<Q")])[0]
+				else:
+					return struct.calcsize("<L")+start, struct.unpack("<L", message[start:start+struct.calcsize("<L")])[0]
 			elif type is types.StringType:
 				length = struct.unpack("<I", message[start:start+intsize])[0]
 				string = message[start+intsize:start+length+intsize]
@@ -188,8 +207,11 @@ class SlskMessage:
 			else:
 				return start, None
 		except struct.error, error:
-			log.addwarning("%s %s" % (self.__class__, error))
-			return start, None
+			log.addwarning("%s %s trying to unpack %s at '%s' at %s/%s" % (self.__class__, error, type, message[start:].__repr__(), self.pos, len(message)))
+			#displayTraceback(sys.exc_info()[2])
+			self.debug(message)
+			raise struct.error, error
+			#return start, None
 
 	def packObject(self, object):
 		""" Returns object (integer, long or string packed into a 
@@ -197,13 +219,19 @@ class SlskMessage:
 		if type(object) is types.IntType:
 			return struct.pack("<i", object)
 		elif type(object) is types.LongType:
-			return struct.pack("<L", object)
+			return struct.pack("<Q", object)
 		elif type(object) is types.StringType:
 			return struct.pack("<i", len(object))+object
 		elif type(object) is types.UnicodeType:
 			log.addwarning(_("Warning: networking thread has to convert unicode string %(object)s message %(type)s") % {'object':object, 'type':self.__class__})
 			encoded = object.encode("utf-8",'replace')
 			return struct.pack("<i", len(encoded))+encoded
+		elif type(object) is NetworkIntType:
+			return struct.pack("<i", object.value)
+		elif type(object) is NetworkLongType:
+			return struct.pack("<L", object.value)
+		elif type(object) is NetworkLongLongType:
+			return struct.pack("<Q", object.value)
 		log.addwarning(_("Warning: unknown object type %s") % type(object) +" "+ ("in message %(type)s") % {'type':self.__class__})
 		return ""
         
@@ -344,19 +372,19 @@ class AddUser(ServerMessage):
 		if len(message[pos:]) > 0:
 			pos, self.status = self.getObject(message, types.IntType, pos)
 			pos, self.avgspeed = self.getObject(message, types.IntType, pos)
-			pos, size1 = self.getObject(message, types.LongType, pos)
-			pos, size2 = self.getObject(message, types.LongType, pos)
-			#self.downloads = (size2 << 32) + size1
-			self.downloadnum = size2 + size1
+			pos, self.downloadnum = self.getObject(message, types.LongType, pos, getsignedint = 1)
+
 			pos, self.files = self.getObject(message, types.IntType, pos)
 			pos, self.dirs = self.getObject(message, types.IntType, pos)
 			if len(message[pos:]) > 0:
 				pos, self.country = self.getObject(message, types.StringType, pos)
 
 class Unknown6(ServerMessage):
-	def __init__(self):
-		pass
-
+	""" Message 6 """
+	def __init__(self, user=None):
+		self.user = user
+	def makeNetworkMessage(self):
+		return self.packObject(self.user)
 	def parseNetworkMessage(self, message):
 		self.debug()
 		pass
@@ -416,18 +444,21 @@ class AckNotifyPrivileges(ServerMessage):
 	def makeNetworkMessage(self):
 		return self.packObject(self.token)
 class JoinPublicRoom(ServerMessage):
+	""" Message 116 """
 	"""We want to join the Public Chat"""
 	def __init__(self, unknown = 0):
 		self.unknown = unknown
 	def makeNetworkMessage(self):
 		return self.packObject(self.unknown)
 class LeavePublicRoom(ServerMessage):
+	""" Message 151 """
 	"""We want to leave the Public Chat"""
 	def __init__(self, unknown = 0):
 		self.unknown = unknown
 	def makeNetworkMessage(self):
 		return self.packObject(self.unknown)
 class PublicRoomMessage(ServerMessage):
+	""" Message 152 """
 	"""The server sends us messages from random chatrooms"""
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
@@ -466,7 +497,8 @@ class JoinRoom(ServerMessage):
 	def __init__(self, room = None, private = None):
 		self.room = room
 		self.private = private
-	
+		self.owner = None
+		self.operators = []
 	def makeNetworkMessage(self):
 		if self.private is not None:
 			return self.packObject(self.room) + self.packObject(self.private)
@@ -474,8 +506,19 @@ class JoinRoom(ServerMessage):
 	
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
-		self.users = self.getUsers(message[pos:])
-	
+		pos1 = pos
+		pos, self.users = self.getUsers(message[pos:])
+		pos = pos1 + pos
+		
+		if len(message[pos:]) > 0:
+			self.private=True
+			pos, self.owner = self.getObject(message, types.StringType, pos)
+		if len(message[pos:]) > 0 and self.private:
+			pos, numops = self.getObject(message, types.IntType, pos)
+			for i in range(numops):
+				pos, operator = self.getObject(message, types.StringType, pos)
+				self.operators.append(operator)
+
 	def getUsers(self, message):
 		pos, numusers = self.getObject(message, types.IntType)
 		users = []
@@ -489,7 +532,6 @@ class JoinRoom(ServerMessage):
 		pos, statslen = self.getObject(message, types.IntType, pos)
 		for i in range(statslen):
 			pos, users[i][2] = self.getObject(message, types.IntType, pos, getsignedint=1)
-		#            pos, users[i][2] = self.getObject(message, types.IntType, pos)
 			pos, users[i][3] = self.getObject(message, types.IntType, pos)
 			pos, users[i][4] = self.getObject(message, types.IntType, pos)
 			pos, users[i][5] = self.getObject(message, types.IntType, pos)
@@ -500,12 +542,14 @@ class JoinRoom(ServerMessage):
 			pos, users[i][7] = self.getObject(message, types.IntType, pos)
 		if len(message[pos:]) > 0:
 			pos, countrylen = self.getObject(message, types.IntType, pos)
-			for i in range(numusers):
+			for i in range(countrylen):
 				pos, users[i][8] = self.getObject(message, types.StringType, pos)
+	
 		usersdict={}
 		for i in users:
 			usersdict[i[0]] = UserData(i[1:])
-		return usersdict
+
+		return pos, usersdict
 
 
 class PrivateRoomUsers(ServerMessage):
@@ -514,10 +558,7 @@ class PrivateRoomUsers(ServerMessage):
 		self.room = room
 		self.numusers = numusers
 		self.users = users
-			
-	#def makeNetworkMessage(self):
-		#return self.packObject(self.room)
-	
+		
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
 		pos, self.numusers = self.getObject(message, types.IntType, pos)
@@ -525,8 +566,7 @@ class PrivateRoomUsers(ServerMessage):
 		for i in range(self.numusers):
 			pos, user = self.getObject(message, types.StringType, pos)
 			self.users.append(user)
-
-	
+		
 class PrivateRoomOwned(ServerMessage):
 	""" We get this when we've created a private room."""
 	def __init__(self, room = None, number = None):
@@ -540,8 +580,6 @@ class PrivateRoomOwned(ServerMessage):
 		for i in range(self.number):
 			pos, user = self.getObject(message, types.StringType, pos)
 			self.operators.append(user)
-		#self.debug()
-	
 		
 class PrivateRoomAddUser(ServerMessage):
 	""" We get / receive this when we add a user to a private room."""
@@ -577,10 +615,9 @@ class PrivateRoomDisown(ServerMessage):
 	
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
-
 		
 class PrivateRoomSomething(ServerMessage):
-	""" We get this when we've removed a user from a private room."""
+	"""Unknown"""
 	def __init__(self, room = None):
 		self.room = room
 
@@ -589,8 +626,7 @@ class PrivateRoomSomething(ServerMessage):
 	
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
-		#pos, self.user = self.getObject(message, types.StringType, pos)
-		#print message[pos:].__repr__()
+		self.debug()
 		
 class PrivateRoomRemoveUser(ServerMessage):
 	""" We get this when we've removed a user from a private room."""
@@ -638,10 +674,7 @@ class PrivateRoomToggle(ServerMessage):
 	def parseNetworkMessage(self, message):
 		# When this is received, we store it in the config, and disable the appropriate menu item
 		pos, self.enabled = 1, bool(ord(message[0]))
-
-
-
-
+		
 class PrivateRoomAddOperator(ServerMessage):
 	""" We send this to add private room operator abilities to a user"""
 	def __init__(self, room = None, user = None):
@@ -678,7 +711,6 @@ class PrivateRoomOperatorAdded(ServerMessage):
 	
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
-		#pos, self.username = self.getObject(message, types.StringType, pos)
 		
 class PrivateRoomOperatorRemoved(ServerMessage):
 	""" We receive this when privateroom operator abilities are removed"""
@@ -693,6 +725,7 @@ class PrivateRoomOperatorRemoved(ServerMessage):
 		#pos, self.username = self.getObject(message, types.StringType, pos)
 		
 class LeaveRoom(ServerMessage):
+	""" Message 15 """
 	""" We send this when we want to leave a room."""
 	def __init__(self, room = None):
 		self.room = room
@@ -705,6 +738,7 @@ class LeaveRoom(ServerMessage):
 
 
 class UserJoinedRoom(ServerMessage):
+	""" Message 16 """
 	""" Server tells us someone has just joined the room."""
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
@@ -719,12 +753,14 @@ class UserJoinedRoom(ServerMessage):
 		self.userdata = UserData(i)
 
 class UserLeftRoom(ServerMessage):
+	""" Message 17 """
 	""" Well, the opposite."""
 	def parseNetworkMessage(self, message):
 		pos, self.room = self.getObject(message, types.StringType)
 		pos, self.username = self.getObject(message, types.StringType, pos)
 
 class RoomTickerState(ServerMessage):
+	""" Message 113 """
 	def __init__(self):
 		self.room = None
 		self.user = None
@@ -739,6 +775,7 @@ class RoomTickerState(ServerMessage):
 			self.msgs[user] = msg
 
 class RoomTickerAdd(ServerMessage):
+	""" Message 114 """
 	def __init__(self):
 		self.room = None
 		self.user = None
@@ -750,6 +787,7 @@ class RoomTickerAdd(ServerMessage):
 		pos, self.msg = self.getObject(message, types.StringType, pos)
 
 class RoomTickerRemove(ServerMessage):
+	""" Message 115 """
 	def __init__(self, room = None):
 		self.user = None
 		self.room = room
@@ -759,6 +797,7 @@ class RoomTickerRemove(ServerMessage):
 		pos, self.user = self.getObject(message, types.StringType, pos)
 
 class RoomTickerSet(ServerMessage):
+	""" Message 116 """
 	def __init__(self, room = None, msg = None):
 		self.room = room
 		if not msg:
@@ -823,7 +862,7 @@ class FileSearch(ServerMessage):
 	""" We send this to the server when we search for something."""
 	""" Server send this to tell us someone is searching for something."""
 	def __init__(self, requestid = None, text = None):
-		self.searchid = requestid
+		self.searchid = NetworkLongType(requestid)
 		self.searchterm = text
 		if text:
 			self.searchterm = ' '.join([x for x in text.split() if x != '-'])
@@ -835,7 +874,7 @@ class FileSearch(ServerMessage):
 		pos, self.user = self.getObject(message, types.StringType)
 		pos, self.searchid = self.getObject(message, types.IntType, pos)
 		pos, self.searchterm = self.getObject(message, types.StringType, pos)
-		
+				
 class WishlistSearch(FileSearch):
 	pass
 
@@ -887,16 +926,12 @@ class GetUserStats(ServerMessage):
 	def parseNetworkMessage(self, message):
 		pos, self.user = self.getObject(message, types.StringType)
 		pos, self.avgspeed = self.getObject(message, types.IntType, pos, getsignedint = 1)
-		#pos, self.downloadnum = self.getObject(message, types.IntType, pos)
-		#pos, self.something = self.getObject(message, types.IntType, pos)
-		pos, size1 = self.getObject(message, types.LongType, pos)
-		pos, size2 = self.getObject(message, types.LongType, pos)
-		#self.downloads = (size2 << 32) + size1
-		self.downloadnum = size2 + size1
+		pos, self.downloadnum = self.getObject(message, types.LongType, pos, getsignedint = 1)
 		pos, self.files = self.getObject(message, types.IntType, pos)
 		pos, self.dirs = self.getObject(message, types.IntType, pos)
 
 class Relogged(ServerMessage):
+	""" Message 41 """
 	""" Server sends this if someone else logged in under our nickname
 	and then disconnects us """
 	def parseNetworkMessage(self, message):
@@ -973,9 +1008,7 @@ class ExactFileSearch(ServerMessage):
 		pos, self.req = self.getObject(message, types.IntType, pos)
 		pos, self.file = self.getObject(message, types.StringType, pos)
 		pos, self.folder = self.getObject(message, types.StringType, pos)
-		pos, size1 = self.getObject(message, types.IntType, pos)
-		pos, size2 = self.getObject(message, types.IntType, pos)
-		self.size = size1 + size2
+		pos, self.size = self.getObject(message, types.LongType, pos, getsignedint = 1)
 		pos, self.checksum = self.getObject(message, types.IntType, pos)
 
 class AdminMessage(ServerMessage):
@@ -992,7 +1025,7 @@ class GlobalUserList(JoinRoom):
 		return ""
 	
 	def parseNetworkMessage(self, message):
-		self.users = self.getUsers(message)
+		pos, self.users = self.getUsers(message)
 
 
 class TunneledMessage(ServerMessage):
@@ -1015,7 +1048,7 @@ class TunneledMessage(ServerMessage):
 		pos, self.msg = self.getObject(message, types.StringType, pos)
 
 class ParentMinSpeed(ServerMessage):
-	# 83
+	""" Message 83 """
 	def __init__(self):
 		pass
 	
@@ -1023,7 +1056,7 @@ class ParentMinSpeed(ServerMessage):
 		pos, self.num = self.getObject(message, types.IntType)
 
 class ParentSpeedRatio(ParentMinSpeed):
-	# 84
+	""" Message 84 """
 	def __init__(self):
 		pass
 	
@@ -1031,7 +1064,7 @@ class ParentSpeedRatio(ParentMinSpeed):
 		pos, self.num = self.getObject(message, types.IntType)
 
 class SearchParent(ServerMessage):
-	# 73
+	""" Message 73 """
 	def __init__(self, parentip = None):
 		self.parentip = parentip
 	
@@ -1093,7 +1126,7 @@ class Msg89(ServerMessage):
 		pass
 
 class DistribAliveInterval(ServerMessage):
-	# 90
+	""" Message 90 """
 	def __init__(self):
 		pass
 	
@@ -1102,6 +1135,7 @@ class DistribAliveInterval(ServerMessage):
 
 
 class WishlistInterval(ServerMessage):
+	""" Message 104"""
 	def __init__(self):
 		pass
 		
@@ -1109,6 +1143,7 @@ class WishlistInterval(ServerMessage):
 		pos, self.seconds = self.getObject(message, types.IntType)
 
 class PrivilegedUsers(ServerMessage):
+	""" Message 69 """
 	""" A list of those who made a donation """
 	def __init__(self):
 		pass
@@ -1126,6 +1161,7 @@ class PrivilegedUsers(ServerMessage):
 			self.users.append(user)
 
 class CheckPrivileges(ServerMessage):
+	""" Message 92 """
 	def __init__(self):
 		pass
 	
@@ -1136,6 +1172,7 @@ class CheckPrivileges(ServerMessage):
 		pos, self.seconds = self.getObject(message, types.IntType)
 
 class AddToPrivileged(ServerMessage):
+	""" Message 91 """
 	def __init__(self):
 		pass
 	
@@ -1143,6 +1180,7 @@ class AddToPrivileged(ServerMessage):
 		l2, self.user = self.getObject(message, types.StringType)
 
 class CantConnectToPeer(ServerMessage):
+	""" Message 1001 """
 	""" We send this to say we can't connect to peer after it has asked us
 	to connect. We receive this if we asked peer to connect and it can't do
 	this. So this message means a connection can't be established either way.
@@ -1157,12 +1195,13 @@ class CantConnectToPeer(ServerMessage):
 	def parseNetworkMessage(self, message):
 		pos, self.token = self.getObject(message, types.IntType)
 
-class CantCreateRoom(ServerMessage):
-	""" Server tells us a new room cannot be created"""
-	def parseNetworkMessage(self, message):
-		self.room = self.getObject(message, types.StringType)[1]
+#class CantCreateRoom(ServerMessage):
+	#""" Server tells us a new room cannot be created"""
+	#def parseNetworkMessage(self, message):
+		#self.room = self.getObject(message, types.StringType)[1]
 
 class ServerPing(ServerMessage):
+	""" Message 32 """
 	def makeNetworkMessage(self):
 		return ""
 		
@@ -1303,7 +1342,7 @@ class RoomSearch(ServerMessage):
 		return self.packObject(self.room)+self.packObject(self.searchid)+self.packObject(self.searchterm)
 	
 	def parseNetworkMessage(self, message):
-		pos, self.user = self.getObject(message, types.StringType)
+		pos, self.room = self.getObject(message, types.StringType)
 		pos, self.searchid = self.getObject(message, types.IntType, pos)
 		pos, self.searchterm = self.getObject(message, types.StringType, pos)
 
@@ -1432,20 +1471,25 @@ class SharedFileList(PeerMessage):
 				self.list={}
 				return
 
-		shares = {}
+		shares = []
 		pos, ndir = self.getObject(message, types.IntType)
 		for i in range(ndir):
 			pos, directory = self.getObject(message, types.StringType, pos)
 			pos, nfiles = self.getObject(message, types.IntType, pos)
-			shares[directory] = []
+			files = []
 			for j in range(nfiles):
 				pos, code = pos+1, ord(message[pos])
 				pos, name = self.getObject(message, types.StringType, pos)
-				pos, size1 = self.getObject(message, types.LongType, pos)
-				pos, size2 = self.getObject(message, types.LongType, pos)
-				#size = (size2 << 32) + size1
-				size = size2 + size1
-		
+				pos, size = self.getObject(message, types.LongType, pos, getsignedint = True)
+				if message[pos-1] == '\xff':
+					# Buggy SLSK?
+					# Some file sizes will be huge if unpacked as a signed
+					# LongType, namely somewhere in the area of 17179869 Terabytes.
+					# It would seem these files are indeed big, but in the Gigabyte range.
+					# The following will undo the damage (and if we fuck up it
+					# doesn't matter, it can never be worse than reporting 17
+					# exabytes for a single file)
+					size = struct.unpack("Q", '\xff'*struct.calcsize("Q"))[0] - size
 				pos, ext = self.getObject(message, types.StringType, pos)
 				pos, numattr = self.getObject(message, types.IntType, pos)
 				attrs = []
@@ -1453,7 +1497,9 @@ class SharedFileList(PeerMessage):
 					pos, attrnum = self.getObject(message, types.IntType, pos)
 					pos, attr = self.getObject(message, types.IntType, pos)
 					attrs.append(attr)
-				shares[directory].append([code, name, size, ext, attrs])
+				files.append([code, name, size, ext, attrs])
+			shares.append((directory, files))
+
 		self.list = shares
 
 	def makeNetworkMessage(self, nozlib = 0, rebuild=False):
@@ -1514,42 +1560,42 @@ class FileSearchResult(PeerMessage):
 		self.ulspeed = ulspeed 
 		self.inqueue = inqueue
 		self.fifoqueue = fifoqueue
-	
+		self.pos = 0
 	def parseNetworkMessage(self, message):
 		try:
 			message = zlib.decompress(message)
+		
+
+			self.pos, self.user = self.getObject(message, types.StringType)
+			self.pos, self.token = self.getObject(message, types.IntType, self.pos)
+			self.pos, nfiles = self.getObject(message, types.IntType, self.pos)
+			shares = []
+			for i in range(nfiles):
+				self.pos, code = self.pos+1, ord(message[self.pos])
+				self.pos, name = self.getObject(message, types.StringType, self.pos)
+				#self.pos, size = self.getObject(message, types.LongType, self.pos, getsignedint=1)
+				self.pos, size2 = self.getObject(message, types.LongType, self.pos)
+				self.pos, size3 = self.getObject(message, types.LongType, self.pos)
+				size = long(size2 + size3)
+
+				self.pos, ext = self.getObject(message, types.StringType, self.pos)
+				self.pos, numattr = self.getObject(message, types.IntType, self.pos)
+				attrs = []
+				if numattr:
+					for j in range(numattr):
+						self.pos, attrnum = self.getObject(message, types.IntType, self.pos)
+						self.pos, attr = self.getObject(message, types.IntType, self.pos)
+						attrs.append(attr)
+				shares.append([code, name, size, ext, attrs])
+			self.list = shares
+			self.pos, self.freeulslots = self.pos+1, ord(message[self.pos])
+			self.pos, self.ulspeed = self.getObject(message, types.IntType, self.pos, getsignedint=1)
+			self.pos, self.inqueue = self.getObject(message, types.IntType, self.pos)
 		except Exception, error:
+			print error
 			log.addwarning(_("Exception during parsing %(area)s: %(exception)s") % {'area':'FileSearchResult', 'exception':error})
 			self.list = {}
 			return
-	
-		pos, self.user = self.getObject(message, types.StringType)
-		pos, self.token = self.getObject(message, types.IntType, pos)
-		pos, nfiles = self.getObject(message, types.IntType, pos)
-		shares = []
-		for i in range(nfiles):
-			size = size1 = size2 = 0
-			pos, code = pos+1, ord(message[pos])
-			pos, name = self.getObject(message, types.StringType, pos)
-			pos, size1 = self.getObject(message, types.LongType, pos)
-			pos, size2 = self.getObject(message, types.LongType, pos)
-			#size = (size2 << 32) + size1
-			size = size2 + size1
-
-			pos, ext = self.getObject(message, types.StringType, pos)
-			pos, numattr = self.getObject(message, types.IntType, pos)
-			attrs = []
-			if numattr:
-				for j in range(numattr):
-					pos, attrnum = self.getObject(message, types.IntType, pos)
-					pos, attr = self.getObject(message, types.IntType, pos)
-					attrs.append(attr)
-			shares.append([code, name, size, ext, attrs])
-		self.list = shares
-		pos, self.freeulslots = pos+1, ord(message[pos])
-		pos, self.ulspeed = self.getObject(message, types.IntType, pos, getsignedint=1)
-		pos, self.inqueue = self.getObject(message, types.IntType, pos)
-        
 	def makeNetworkMessage(self):
 		filelist = []
 		for i in self.list:
@@ -1574,12 +1620,15 @@ class FileSearchResult(PeerMessage):
 
 		msg = self.packObject(self.user)+self.packObject(self.token)+self.packObject(len(filelist))
 		for i in filelist:
-			msg = msg+chr(1)+self.packObject(i[0].replace(os.sep,"\\"))+self.packObject(i[1])+self.packObject(0)
+			# Code, Filename, Size (Long Long), 
+			msg = msg+chr(1)+self.packObject(i[0].replace(os.sep,"\\"))+self.packObject(i[1])
 			if i[2] is None:
 				# No metadata
 				msg = msg + self.packObject('')+self.packObject(0)
 			else:
+				#FileExtension, NumAttributes, 
 				msg = msg + self.packObject("mp3") + self.packObject(3)
+				#AttributeNum 0, Bitrate, AttributeNum 1, Length, AttributeNum 2, Variable Bitrate,
 				msg = msg + self.packObject(0) + self.packObject(i[2][0])+self.packObject(1)+ self.packObject(i[3])+self.packObject(2)+self.packObject(i[2][1])
 		msg = msg+chr(self.freeulslots)+self.packObject(self.ulspeed)+self.packObject(queuesize)
 		return zlib.compress(msg)
@@ -1627,13 +1676,9 @@ class FolderContentsResponse(PeerMessage):
 				pos, nfiles = self.getObject(message, types.IntType, pos)
 				shares[folder][directory] = []
 				for j in range(nfiles):
-					size = size1 = size2 = 0
 					pos, code = pos+1, ord(message[pos])
 					pos, name = self.getObject(message, types.StringType, pos)
-					pos, size1 = self.getObject(message, types.LongType, pos)
-					pos, size2 = self.getObject(message, types.LongType, pos)
-					#size = (size2 << 32) + size1
-					size = size2 + size1
+					pos, size = self.getObject(message, types.LongType, pos, getsignedint = 1)
 					pos, ext = self.getObject(message, types.StringType, pos)
 					pos, numattr = self.getObject(message, types.IntType, pos)
 					attrs = []
@@ -1669,6 +1714,7 @@ class TransferRequest(PeerMessage):
 	def makeNetworkMessage(self):
 		msg = self.packObject(self.direction)+self.packObject(self.req)+self.packObject(self.file)
 		if self.filesize is not None and self.direction == 1:
+			# BORKED Uploads to the official client: msg = msg+self.packObject(self.filesize)
 			msg = msg+self.packObject(self.filesize) + self.packObject(0)
 		return msg
 	
@@ -1695,6 +1741,7 @@ class TransferResponse(PeerMessage):
 		if self.reason is not None:
 			msg = msg + self.packObject(self.reason)
 		if self.filesize is not None:
+			# BORKED Uploads to the official client: msg = msg + self.packObject(self.filesize)
 			msg = msg + self.packObject(self.filesize) + self.packObject(0)
 		return msg
 	
@@ -1766,7 +1813,7 @@ class FileRequest(PeerMessage):
 		msg = self.packObject(self.req)
 		return msg
 
-class HaveNoParent(ServerMessage):
+class HaveNoParent(ServerMessage): #71
 	def __init__(self, noparent = None):
 		self.noparent = noparent
 	
