@@ -110,6 +110,7 @@ class PeerConnection(Connection):
 	
 		self.starttime = None # Used for upload bandwidth management
 		self.sentbytes2 = 0
+		self.readbytes2 = 0
 
 
 class PeerConnectionInProgress:
@@ -120,7 +121,7 @@ class PeerConnectionInProgress:
 	def __init__(self, conn = None, msgObj = None):
 		self.conn = conn
 		self.msgObj = msgObj
-		#self.lastactive = time.time()
+		self.lastactive = time.time()
 		
 class SlskProtoThread(threading.Thread):
 	""" This is a netwroking thread that actually does all the communication.
@@ -251,7 +252,12 @@ class SlskProtoThread(threading.Thread):
 
 	distribclasses = {0:DistribAlive, 3:DistribSearch, 4:DistribBranchLevel, 5: DistribBranchRoot, 7: DistribChildDepth, 9:DistribMessage9}
 
-	
+	IN_PROGRESS_STALE_AFTER = 30
+	# The value of 30 was pulled out of thin air. When we let the OS handle this:
+	# - Linux seems okay, stale in progress conns get killed after a minute or two
+	# - With Windows, based on #473, it would seem these connections are never removed
+	CONNECTION_MAX_IDLE = 60
+
 	def __init__(self, ui_callback, queue, config, eventprocessor):
 		""" ui_callback is a UI callback function to be called with messages 
 		list as a parameter. queue is Queue object that holds messages from UI
@@ -276,7 +282,9 @@ class SlskProtoThread(threading.Thread):
 		self._conns = {}
 		self._connsinprogress = {}
 		self._uploadlimit = (self._calcLimitNone, 0)
+		self._downloadlimit = (self._calcDLimitByTotal, self._config.sections["transfers"]["downloadlimit"])
 		self._limits = {}
+		self._dlimits = {}
 		# GeoIP Config
 		self._geoip = None
 		# GeoIP Module
@@ -304,7 +312,10 @@ class SlskProtoThread(threading.Thread):
 	
 	def _isUpload(self, conn):
 		return conn.__class__ is PeerConnection and conn.fileupl is not None
-	
+	def _isDownload(self, conn):
+		return conn.__class__ is PeerConnection and conn.filedown is not None
+
+
 	def _calcUploadSpeed(self, i):
 		curtime = time.time()
 		if i.starttime is None:
@@ -332,7 +343,28 @@ class SlskProtoThread(threading.Thread):
 		if limit < 1024.0:
 			return long(0)
 		return long(limit)
-	
+
+	def _calcDownloadSpeed(self, i):
+		curtime = time.time()
+		if i.starttime is None:
+			i.starttime = curtime
+		elapsed = curtime - i.starttime
+		if elapsed == 0:
+			return 0
+		else:
+			return i.readbytes2 / elapsed
+
+	def _calcDLimitByTotal(self, conns, i):
+		max = self._downloadlimit[1] * 1024.0
+		bw = 0.0
+		for j in conns:
+			if self._isDownload(j):
+				bw += self._calcDownloadSpeed(j)
+		limit = max - bw + 1023
+		if limit < 1024.0:
+			return long(0)
+		return long(limit)
+
 	def _calcLimitNone(self, conns, i):
 		return None
 	
@@ -356,12 +388,14 @@ class SlskProtoThread(threading.Thread):
 			outsocks = [i for i in conns.keys() if len(conns[i].obuf) > 0 or (i is not server_socket and conns[i].fileupl is not None and conns[i].fileupl.offset is not None)]
 			outsock = []
 			self._limits = {}
+			self._dlimits = {}
 			for i in outsocks:
 				if self._isUpload(conns[i]):
 					limit = self._uploadlimit[0](conns, conns[i])
 					if limit is None or limit > 0:
 						self._limits[i] = limit
 						outsock.append(i)
+				
 				else:
 					outsock.append(i)
 			try:
@@ -376,7 +410,7 @@ class SlskProtoThread(threading.Thread):
 				numsockets += len(conns) + len(connsinprogress)
 
 				self._ui_callback([InternalData(numsockets)])
-				#print "Sockets open:", len(conns.keys()+connsinprogress.keys()+[p]+outsock), len(conns.keys()),  len(connsinprogress.keys())
+				#print "Sockets open: %s = %s + %s + %s (+1)" % (len(conns.keys()+connsinprogress.keys()+[p]+outsock), len(conns.keys()),  len(connsinprogress.keys()), len(outsock))
 			except select.error, error:
 				if len(error.args) == 2 and error.args[0] == EINTR:
 					# Error recieved; but we don't care :)
@@ -415,6 +449,10 @@ class SlskProtoThread(threading.Thread):
 			# Manage Connections
 			curtime = time.time()
 			for connection_in_progress in connsinprogress.keys()[:]:
+				if (curtime - connsinprogress[connection_in_progress].lastactive) > self.IN_PROGRESS_STALE_AFTER:
+					connection_in_progress.close()
+					del connsinprogress[connection_in_progress]
+					continue
 				try:
 					msgObj = connsinprogress[connection_in_progress].msgObj
 					if connection_in_progress in input:
@@ -434,12 +472,9 @@ class SlskProtoThread(threading.Thread):
 								message = "Blocking peer connection in progress to IP: %(ip)s Port: %(port)s" % { "ip":ip, "port":port}
 								self._ui_callback([DebugMessage(message, 3)])
 								connection_in_progress.close()
-				
 							else:
 								conns[connection_in_progress] = PeerConnection(connection_in_progress, msgObj.addr, "", "", msgObj.init)
 								self._ui_callback([OutConn(connection_in_progress, msgObj.addr)])
-							
-						
 						del connsinprogress[connection_in_progress]
 			# Process Data
 			for connection in conns.keys()[:]:
@@ -453,10 +488,23 @@ class SlskProtoThread(threading.Thread):
 					continue
 
 				if connection in input:
-					try:
-						self.readData(conns, connection)
-					except socket.error, err:
-						self._ui_callback([ConnectError(conns[connection], err)])
+					if self._isDownload(conns[connection]):
+						limit = self._downloadlimit[0](conns, connection)
+						if limit is None or limit > 0:
+							self._dlimits[connection] = limit
+							#if connection in self._dlimits:
+								#Todo: fix this Ugly download limit hack (sleep)
+								#time.sleep(1.0)
+								
+						try:
+							self.readData(conns, connection)
+						except socket.error, err:
+							self._ui_callback([ConnectError(conns[connection], err)])
+					else:
+						try:
+							self.readData(conns, connection)
+						except socket.error, err:
+							self._ui_callback([ConnectError(conns[connection], err)])
 				if connection in conns and len(conns[connection].ibuf) > 0:
 					if connection is server_socket:
 						msgs, conns[server_socket].ibuf = self.process_server_input(conns[server_socket].ibuf)
@@ -480,37 +528,29 @@ class SlskProtoThread(threading.Thread):
 			# Timeout Connections
 			curtime = time.time()
 			connsockets = len(conns.keys())
-			num = 0
 			for connection in conns.keys()[:]:
 				if connection is not server_socket and connection is not p:
-					if curtime - conns[connection].lastactive > 60:
-
+					if curtime - conns[connection].lastactive > self.CONNECTION_MAX_IDLE:
 						self._ui_callback([ConnClose(connection, conns[connection].addr)])
 						connection.close()
 						#print "Closed_run", conns[i].addr
 						del conns[connection]
-			
 					#  Was 30 seconds
-			
-				num += 1
 			if server_socket in conns.keys():
 				if curtime - conns[server_socket].lastping > 120:
 					conns[server_socket].lastping = curtime
 					queue.put(ServerPing())
 					
 			self._ui_callback([])
+			if self._downloadlimit[1] and self._downloadlimit[1] > 0:
+				time.sleep(0.1)
 
 		# Close Server Port
 		if server_socket is not None:
 			server_socket.close()
+		#print "Networking thread aborted"
 		self._stopped = 1
 		
-	def checkTimeSinceActive(self, conn):
-		for i in self._conns.keys():
-			if i == conn:
-				return self._conns[conn].lastactive
-		return None
-		
 	def socketStillActive(self, conn):
 		for i in self._conns.keys():
 			if i == conn:
@@ -585,11 +625,24 @@ class SlskProtoThread(threading.Thread):
 					self._ui_callback([conns[i].fileupl])
 
 	def readData(self, conns, i):
+		# Check for a download limit
+		if i in self._dlimits:
+			limit = self._dlimits[i]
+		else:
+			limit = None
 		conns[i].lastactive = time.time()
-		data = i.recv(conns[i].lastreadlength)
-		conns[i].ibuf = conns[i].ibuf + data
-		if len(data) >= conns[i].lastreadlength//2:
-			conns[i].lastreadlength = conns[i].lastreadlength * 2 
+		if limit is None:
+			# Unlimited download data
+			data = i.recv(conns[i].lastreadlength)
+			conns[i].ibuf = conns[i].ibuf + data
+			if len(data) >= conns[i].lastreadlength//2:
+				conns[i].lastreadlength = conns[i].lastreadlength * 2 
+		else:
+			# Speed Limited Download data (transfers)
+			data = i.recv(conns[i].lastreadlength )
+			conns[i].ibuf += data
+			conns[i].lastreadlength = limit
+			conns[i].readbytes2 += len(data)
 		if not data:
 			self._ui_callback([ConnClose(i, conns[i].addr)])
 			i.close()
@@ -728,13 +781,14 @@ class SlskProtoThread(threading.Thread):
 							host = port = _("unknown")
 							msgname = str(self.peerclasses[msgtype]).split(".")[-1]
 							print "Error parsing %s:" % msgname, error
-							if "addr" in conn.init.conn.__dict__:
-								if conn.init.conn.addr is not None:
-									host = conn.init.conn.addr[0]
-									port = conn.init.conn.addr[1]
+							if "addr" in conn.__dict__:
+								if conn.addr is not None:
+									host = conn.addr[0]
+									port = conn.addr[1]
 							debugmessage = _("There was an error while unpacking Peer message type %(type)s size %(size)i contents %(msgBuffer)s from user: %(user)s, %(host)s:%(port)s") %{'type':msgname, 'size':msgsize-4, 'msgBuffer':msgBuffer[8:msgsize+4].__repr__(), 'user':conn.init.user, 'host': host, 'port': port}
 							print debugmessage
 							msgs.append(debugmessage)
+							del msg
 						else:
 							msgs.append(msg)
 					except Exception, error:
@@ -806,6 +860,9 @@ class SlskProtoThread(threading.Thread):
 			if self._isUpload(i):
 				i.starttime = curtime
 				i.sentbytes2 = 0
+			if self._isDownload(i):
+				i.starttime = curtime
+				i.sentbytes2 = 0
 
 	def process_queue(self, queue, conns, connsinprogress, server_socket, maxsockets=MAXFILELIMIT):
 		""" Processes messages sent by UI thread. server_socket is a server connection
@@ -813,7 +870,7 @@ class SlskProtoThread(threading.Thread):
 		are dictionaries holding Connection and PeerConnectionInProgress 
 		messages."""
 		msgList = []
-		needsleep = 0
+		needsleep = False
 		numsockets = 0
 		if server_socket is not None:
 			numsockets += 1
@@ -828,7 +885,7 @@ class SlskProtoThread(threading.Thread):
 						conns[server_socket].obuf = conns[server_socket].obuf + struct.pack("<ii", len(msg)+4, self.servercodes[msgObj.__class__]) + msg
 					else:
 						queue.put(msgObj)
-						needsleep = 1
+						needsleep = True
 				except Exception, error:
 					self._ui_callback([_("Error packaging message: %(type)s %(msg_obj)s, %(error)s") %{'type':msgObj.__class__, 'msg_obj':vars(msgObj), 'error': str(error)}])
 			elif issubclass(msgObj.__class__, PeerMessage):
@@ -899,7 +956,7 @@ class SlskProtoThread(threading.Thread):
 							import errno
 							if errno.errorcode.get(errnum,"") == 'EMFILE':
 								queue.put(msgObj)
-								needsleep = 1
+								needsleep = True
 							else:
 								self._ui_callback([ConnectError(msgObj, (errnum, strerror))])
 					else:
@@ -924,6 +981,8 @@ class SlskProtoThread(threading.Thread):
 						cb = self._calcLimitNone
 					self._resetCounters(conns)
 					self._uploadlimit = (cb, msgObj.limit)
+				elif msgObj.__class__ is SetDownloadLimit:
+					self._downloadlimit = (self._calcDLimitByTotal, msgObj.limit)
 				if socketwarning and time.time() - self.lastsocketwarning > 60:
 					self.lastsocketwarning = time.time()
 					log.addwarning(_("You have just hit your connection limit of %(limit)s. Nicotine+ will drop connections for your protection. If you get this message often you should search for less generic terms, or increase your per-process file descriptor limit.") % {'limit':maxsockets})
