@@ -171,7 +171,7 @@ class Transfers:
 					if i.status not in ["Queued", 'Aborted', 'Cannot connect', 'Paused' ]:
 						self.getFile(i.user, i.filename, i.path, i)
 				else:
-					if i.status not in ['Aborted']:
+					if i.status not in ['Aborted', 'Filtered']:
 						i.status = "User logged off"
 						self.downloadspanel.update(i)
 
@@ -190,7 +190,7 @@ class Transfers:
 
 
 	def getFile(self, user, filename, path="", transfer = None, size=None, bitrate=None, length=None):
-		path=self.CleanPath(path, absolute=True)
+		path=utils.CleanPath(path, absolute=True)
 		self.transferFile(0, user, filename, path, transfer, size, bitrate, length)
 
 	def pushFile(self, user, filename, path="", transfer = None, size=None, bitrate=None, length=None ):
@@ -704,9 +704,9 @@ class Transfers:
 					i.transfertimer.cancel()
 				if not incompletedir:
 					if i.path and i.path[0] == '/':
-						incompletedir = self.CleanPath(i.path)
+						incompletedir = utils.CleanPath(i.path)
 					else:
-						incompletedir = os.path.join(downloaddir, self.CleanPath(i.path))
+						incompletedir = os.path.join(downloaddir, utils.CleanPath(i.path))
 				incompletedir = self.encode(incompletedir, i.user)
 				try:
 					if not os.access(incompletedir, os.F_OK):
@@ -832,20 +832,6 @@ class Transfers:
 			return
 		tablabel.set_image(frame.images["online"])
 		
-	def CleanPath(self, path, absolute=False):
-		if win32:
-			# Without hacks it is (up to Vista) not possible to have more
-			# than 26 drives mounted, so we can assume a '[a-zA-Z]:\' prefix
-			# for drives - we shouldn't escape that
-			drive = ''
-			if absolute and path[1:3] == ':\\' and path[0:1] and path[0].isalpha():
-				drive = path[:3]
-				path = path[3:]
-			chars = ["?", "\"", ":", ">", "<", "|", "*"]
-			for char in chars:
-				path = path.replace(char, "_")
-			path = ''.join([drive, path])
-		return path
 	def FileDownload(self, msg):
 		""" A file download is in progress"""
 		needupdate = 1
@@ -883,10 +869,10 @@ class Transfers:
 					i.status = "Transferring"
 				else:
 					msg.file.close()
-					basename = self.CleanPath(self.encode(string.split(i.filename,'\\')[-1], i.user))
+					basename = utils.CleanPath(self.encode(string.split(i.filename,'\\')[-1], i.user))
 					downloaddir = config["transfers"]["downloaddir"]
 					if i.path and i.path[0] == '/':
-						folder = self.CleanPath(i.path)
+						folder = utils.CleanPath(i.path)
 					else:
 						folder = os.path.join(downloaddir, self.encode(i.path))
 					if not os.access(folder, os.F_OK):
@@ -918,7 +904,7 @@ class Transfers:
 							self.addToShared(u"%s" % newname)
 						else:
 							self.addToShared(newname)
-						self.eventprocessor.sendNumSharedFoldersFiles()
+						self.eventprocessor.shares.sendNumSharedFoldersFiles()
 					self.SaveDownloads()
 					self.downloadspanel.update(i)
 					if config["transfers"]["shownotification"]:
@@ -955,78 +941,9 @@ class Transfers:
 				self.downloadspanel.update(i)
     
 	def addToShared(self, name):
+		self.eventprocessor.shares.addToShared(name)
 		""" Add a file to the normal shares database """
-		config = self.eventprocessor.config.sections
-		if not config["transfers"]["sharedownloaddir"]:
-			return
-		
-		shared = config["transfers"]["sharedfiles"]
-		sharedstreams = config["transfers"]["sharedfilesstreams"]
-		wordindex = config["transfers"]["wordindex"]
-		fileindex = config["transfers"]["fileindex"]
-		shareddirs = config["transfers"]["shared"] + [config["transfers"]["downloaddir"]]
-		sharedmtimes = config["transfers"]["sharedmtimes"]
-
-		dir = str(os.path.expanduser(os.path.dirname(name)))
-		str_name = str(name)
-		file = str(os.path.basename(name))
-		size = os.path.getsize(name)
-
-		shared[dir] = shared.get(dir, [])
-
-		if file not in [i[0] for i in shared[dir]]:
-			fileinfo = utils.getFileInfo(file, name)
-			shared[dir] = shared[dir] + [fileinfo]
-			sharedstreams[dir] = utils.getDirStream(shared[dir])
-			words = utils.getIndexWords(dir, file, shareddirs)
-			self.addToIndex(wordindex, fileindex, words, dir, fileinfo)
-			sharedmtimes[dir] = os.path.getmtime(dir)
-			self.eventprocessor.newnormalshares = True
-			
-		if config["transfers"]["enablebuddyshares"]:
-			self.addToBuddyShared(name)
-			
-		self.eventprocessor.config.writeShares()
-		
-	def addToBuddyShared(self, name):
-		""" Add a file to the buddy shares database """
-		config = self.eventprocessor.config.sections
-		if not config["transfers"]["sharedownloaddir"]:
-			return
-		bshared = config["transfers"]["bsharedfiles"]
-		bsharedstreams = config["transfers"]["bsharedfilesstreams"]
-		bwordindex = config["transfers"]["bwordindex"]
-		bfileindex = config["transfers"]["bfileindex"]
-		bshareddirs = config["transfers"]["buddyshared"] + config["transfers"]["shared"] + [config["transfers"]["downloaddir"]]
-		bsharedmtimes = config["transfers"]["bsharedmtimes"]
-		
-		dir = str(os.path.expanduser(os.path.dirname(name)))
-		str_name = str(name)
-		file = str(os.path.basename(name))
-		size = os.path.getsize(name)
-		
-		bshared[dir] = bshared.get(dir, [])
-		
-		if file not in [i[0] for i in bshared[dir]]:
-			fileinfo = utils.getFileInfo(file, name)
-			bshared[dir] = bshared[dir] + [fileinfo]
-			bsharedstreams[dir] = utils.getDirStream(bshared[dir])
-			words = utils.getIndexWords(dir, file, bshareddirs)
-			self.addToIndex(bwordindex, bfileindex, words, dir, fileinfo)
-			bsharedmtimes[dir] = os.path.getmtime(dir)
-			
-			self.eventprocessor.newbuddyshares = True
-			
-
-	def addToIndex(self, wordindex, fileindex, words, dir, fileinfo):
-		index = len(fileindex.keys())
-		for i in words:
-			if i not in wordindex:
-				wordindex[i] = [index]
-			else:
-				wordindex[i] = wordindex[i] + [index]
-		fileindex[str(index)] = (os.path.join(dir, fileinfo[0]),)+fileinfo[1:]
-
+	
 	def FileUpload(self, msg):
 		""" A file upload is in progress"""
 		needupdate = 1
@@ -1178,7 +1095,21 @@ class Transfers:
 		for i in self.peerconns:
 			if i.conn is msg.conn.conn:
 				user = i.username
-			
+		def listUsers():
+			users = []
+			for i in self.uploads:
+				if i.user not in users:
+					users.append(i.user)
+			return users
+
+		def countTransfers(username):
+			transfers = []
+			for i in self.uploads:
+				if i.status == "Queued":
+					if i.user == username:
+						transfers.append(i)
+			return len(transfers)
+
 		if self.eventprocessor.config.sections["transfers"]["fifoqueue"]:
 			# Number of transfers queued by non-privileged users
 			count = 0
@@ -1207,41 +1138,44 @@ class Transfers:
 			# Debugging
 			#print i.user, i.filename, count, countpriv, place
 		else:
-
+			# Todo
 			list = listogg = listpriv = {user:time.time()}
 			countogg = 0
 			countpriv = 0
-			
+			trusers = self.getTransferringUsers()
+			count = 0
+			place = 0
+			transfers = 0
 			for i in self.uploads:
 				# Ignore non-queued files
 				if i.status == "Queued":
-					if i.user in listpriv.keys() or self.isPrivileged(i.user):
-						listpriv[i.user] = i.timequeued
-						countpriv += 1
-					elif i.filename[-4:].lower() == ".ogg":
-						listogg[i.user] = i.timequeued
-						countogg += 1
-					else:
-						list[i.user] = i.timequeued
-			place = 0
-			if self.isPrivileged(user):
-				# Only have priv'd files in the list
-				list = listpriv
-			elif msg.file[-4:].lower() == ".ogg":
-				# Only have ogg files in the list
-				list = listogg
-				# Add priv'd numbers to place, since
-				# They won't be in the list
-				place += countpriv
-			else:
-				# If file is not ogg and user is not privileged
-				# Add priv'd and ogg numbers to place, since
-				# They won't be in the list
-				place += countpriv + countogg
-				
-			for i in list.keys():
-				if list[i] < list[user]:
-					place += 1
+					if i.user == user:
+						if self.isPrivileged(user):
+							# User is privileged so we only 
+							# count priv'd transfers
+							listpriv[i.user] = i.timequeued
+							place += 1
+						else:
+							# Count all transfers
+							place += 1 
+						# Stop counting on the matching file
+						if i.filename == msg.file:
+							break
+			uploadUsers = listUsers()
+			userTransfers = {}
+			for username in uploadUsers:
+				userTransfers[username] = countTransfers(username)
+				if username is not user:
+					if userTransfers[username] >= place:
+						if username not in trusers:
+							#print username, place
+							transfers += place
+					#else:
+						#transfers += userTransfers[username]
+			#print userTransfers, place, transfers
+			place += transfers
+			#print place
+			#print trusers
 	
 		self.queue.put(slskmessages.PlaceInQueue(msg.conn.conn, msg.file, place))
 
@@ -1364,6 +1298,10 @@ class Transfers:
 				if i.user in self.users and self.users[i.user].status == 0:
 					i.status = "User logged off"
 				else:
+					#if i.status == 'Initializing transfer':
+					#	illegal = set(utils.illegalwinchars).intersection(i.filename)
+					#	if len(illegal) > 0:
+					#		log.add(_("File transfer of %(file)s might have failed because of the %(chars)s characters in the name and because the other client is buggy.") % {'file':self.decode(i.filename), 'chars':''.join(illegal)})
 					i.status = "Connection closed by peer"
 					if i in self.downloads:
 						self.eventprocessor.logTransfer(_("Retrying failed download: %(user)s, file %(file)s") % {'user':i.user, 'file':self.decode(i.filename)}, 1)
@@ -1490,8 +1428,8 @@ class Transfers:
 					
 					if self.eventprocessor.config.sections["transfers"]["prioritize"]:
 						for file in msg.list[i][directory]:
-							(junk, sep, ext) = file[1].rpartition('.')
-							if sep and ext in ['sfv','md5','nfo']:
+							parts = file[1].rsplit('.', 1)
+							if len(parts) == 2 and parts[1] in ['sfv','md5','nfo']:
 								priorityfiles.append(file)
 							else:
 								normalfiles.append(file)
@@ -1569,7 +1507,7 @@ class Transfers:
 	def SaveDownloads(self):
 		""" Save list of files to be downloaded """
 		self.eventprocessor.config.sections["transfers"]["downloads"] = self.GetDownloads()
-		self.eventprocessor.config.writeConfig()
+		self.eventprocessor.config.writeTransfers()
 	
 	def decode(self, string):
 		try:
